//============================================================================================!
//                                                                                            !
//  FluidProp      Teus van der Stelt        Copyright  2004-2014.  All Rights Reserved.      !
//  =========      Piero Colonna             Energy Technology Section, TU Delft.             !
//                                                                                            !
//                                                                                            !
//                                                                                            !
//  Module name   : FluidProp                                                                 !
//                                                                                            !   
//  Description   : This module loads and provides access to the FLUIDPROP library.           !
//                                                                                            !   
//  Contents      : Procedures for object initialization and cleanup:                         !   
//                  1.  FLUIDPROP_IsInit                                                      !   
//                  2.  Init_FLUIDPROP                                                        !   
//                  3.  UnInit_FLUIDPROP                                                      !   
//                                                                                            !   
//                  Procedures according the FLUIDPROP interface:                             !   
//                  4. SubLibTest                                                             !
//                                                                                            !
//  Creation date : 21-07-2008, Henk Seubers, VORtech                                         !
//                                                                                            !   
//============================================================================================!

#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include "config.h"
#include "shared.h"
#include "fstring.h"
#include "fluidprop.h"

// Library handle (NULL in case of uninitialized) 
static void *hLibrary = NULL;
static void *hGasMix = NULL;
static void *hIF97 = NULL;
static void *hRefProp = NULL;
static void *hfreeStanMix = NULL;
static void *hTPSI = NULL;
static void *hvThermo = NULL;
static void *hStanMix = NULL;
static void *hPCP_SAFT = NULL;
static void *hScalingLaws = NULL;

// define the required length of the error message buffer
static const SIZE_TYPE len_ErrorMsg = 255;
static char ErrorMsg[256];

// Library function signatures
typedef void (FP_SO_CALL *TSubLibTest)();

typedef void (FP_SO_CALL *TCreateObject)( const char* ModelName, char* ErrorMsg, SIZE_TYPE len_ModelName, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TGetVersion)( const char* ModelName, int32_t* version, SIZE_TYPE len_ModelName);

typedef void (FP_SO_CALL *TReleaseObjects)(void);

typedef void (FP_SO_CALL *TSetFluid)( const char* ModelName, int32_t* nComp, const char* Comp, const double* Conc, char* ErrorMsg, SIZE_TYPE len_ModelName, 
                                      SIZE_TYPE len_Comp, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TGetFluid)( char* ModelName, int32_t* nComp, char* Comp, double* Conc, SIZE_TYPE len_ModelName, SIZE_TYPE len_Comp);

typedef void (FP_SO_CALL *TSetTable)( const char* TableName, const char* TableType, const char* InterpolMethod, 
				      const char* MeshName, const char* SearchAlg, int32_t* phase, char* ErrorMsg, 
                                      SIZE_TYPE len_TableName, SIZE_TYPE len_TableType, 
				      SIZE_TYPE len_InterpolMethod, SIZE_TYPE len_MeshName, 
				      SIZE_TYPE len_SearchAlg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSetTable_New)( const char* TableName, const char* InputSpec, double* Input1_min, 
                                            double* Input1_max, double* Input2_min, double* Input2_max, 
                                            double* MeshSpacing, char* TableType, char* IntPolMethod, char* SearchAlg, 
                                            char* ErrorMsg, SIZE_TYPE len_TableName, SIZE_TYPE len_InputSpec, 
                                            SIZE_TYPE len_TableType, SIZE_TYPE len_IntPolMethod, SIZE_TYPE len_SearchAlg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TUseTable)( const char* TableName, char* ErrorMsg, 
                                      SIZE_TYPE len_TableName, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TGetFluidNames)( const char* LongShort, const char* ModelName, int32_t* nFluids, 
                                           char* FluidNames, char* ErrorMsg, SIZE_TYPE len_LongShort, SIZE_TYPE len_ModelName, 
                                           SIZE_TYPE len_FluidNames, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllPropsSat)( const char* InputSpec, double* Input1,   
                                                        double* Input2, double* P, double* T, double* v, double* d, double* h, double* s, double* u, double* q,     
                                                        double* x, double* y, double* cv, double* cp, double* c, double* alpha, double* beta, 
                                                        double* chi, double* fi, double* ksi, double* psi, double* zeta, double* theta,     
                                                        double* kappa, double* gamma, double* eta, double* lambda, double* d_liq,   
                                                        double* d_vap, double* h_liq, double* h_vap, double* T_sat,         
                                                        double* dd_liq_dP, double* dd_vap_dP, double* dh_liq_dP,    
                                                        double* dh_vap_dP, double* dT_sat_dP, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllProps)( const char* InputSpec, double* Input1,   double* Input2, double* P, double* T, double* v, double* d, double* h, 
                                      double* s, double* u, double* q, double* x, double* y, double* cv, double* cp, double* c, double* alpha, double* beta, 
                                      double* chi, double* fi, double* ksi, double* psi, double* zeta, double* theta, double* kappa, double* gamma, double* eta, 
                                      double* lambda, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllPropsSU2)( const char* InputSpec, double* Input1, double* Input2, double* P, double* T, double* d, 
                                         double* s, double* u, double* cv, double* c2, double* alpha, double* beta, char* ErrorMsg, 
                                         SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllPropsJoe)( const char* InputSpec, double* Input1,   double* Input2, double* P, double* T, double* v, double* d, double* h, 
                                      double* s, double* u, double* cv, double* cp, double* c, double* alpha, double* beta, 
                                      double* zeta, double* eta, double* lambda, double* dlam_dT, double* dlam_drho, double* deta_dT, double* deta_drho, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSolve)( const char* FuncSpec, double* FuncVal, const char* InputSpec, int* TargetProp, double* FixedVal,       
                                     double* MinVal, double* MaxVal, char* ErrorMsg, SIZE_TYPE len_FuncSpec, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TPressure)( const char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TTemperature)( const char* InputSpec, double* Input1,   double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSpecVolume)( const char* InputSpec, double* Input1,    double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TDensity)( const char* InputSpec, double* Input1,       double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TEnthalpy)( const char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TEntropy)( const char* InputSpec, double* Input1,       double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TIntEnergy)( const char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TVaporQual)( const char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TLiquidCmp)( const char* InputSpec, double* Input1,     double* Input2,  double* Output, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TVaporCmp)( const char* InputSpec, double* Input1,      double* Input2,  double* Output, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *THeatCapV)( const char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *THeatCapP)( const char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSoundSpeed)( const char* InputSpec, double* Input1,    double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TAlpha)( const char* InputSpec, double* Input1,         double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TBeta)( const char* InputSpec, double* Input1,          double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TChi)( const char* InputSpec, double* Input1,           double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TFi)( const char* InputSpec, double* Input1,            double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TKsi)( const char* InputSpec, double* Input1,           double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TPsi)( const char* InputSpec, double* Input1,           double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TZeta)( const char* InputSpec, double* Input1,          double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TTheta)( const char* InputSpec, double* Input1,         double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TKappa)( const char* InputSpec, double* Input1,         double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TGamma)( const char* InputSpec, double* Input1,         double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TViscosity)( const char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TThermCond)( const char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSurfTens)( const char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TMmol) ( char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TTcrit)( char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TPcrit)( char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TTmin) ( char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TTmax) ( char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllInfo)( double* M_mol, double* T_crit, double* P_crit, double* T_min, double* T_max, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSetUnits)( const char* UnitSet, const char* MassOrMole, const char* Properties, const char* Units, char* ErrorMsg, 
                     SIZE_TYPE len_UnitSet, SIZE_TYPE len_MassOrMole, SIZE_TYPE len_Properties, SIZE_TYPE len_Units, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSetRefState)(double* T_ref, double* P_ref, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSetRefState_NonDim)(double* T_ref, double* P_ref, double* v_ref, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TzFlow_vu)( double* P, double* T, double* v, double* h, double* s, double* u, double* alpha, double* beta, double* chi, double* fi, double* zeta, double* gamma,
                                                     char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSaturationLine) ( int* nPnts, double* T, double* P, double* x, double* y, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllTransProps)( const char* InputSpec, double* Input1, double* Input2, double* eta, double* eta_T, double* eta_rho, 
                                           double* lambda,double* lambda_T, double* lamba_rho, double* sigma, char* ErrorMsg, 
                                           SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

// Library function handles
static TSubLibTest hSubLibTest = NULL;
static TCreateObject hcreateobject = NULL;
static TGetVersion hgetversion = NULL;
static TReleaseObjects hreleaseobjects = NULL;
static TSetFluid hsetfluid = NULL;
static TGetFluid hgetfluid = NULL;
static TSetTable hsettable = NULL;
static TSetTable_New hsettable_new = NULL;
static TUseTable husetable = NULL;
static TGetFluidNames hgetfluidnames = NULL;
static TAllPropsSat hallpropssat = NULL;
static TAllProps hallprops = NULL;
static TAllPropsSU2 hallpropssu2 = NULL;
static TAllPropsJoe hallpropsjoe = NULL;
static TSolve hsolve = NULL;
static TPressure hpressure = NULL;
static TTemperature htemperature = NULL;
static TSpecVolume hspecvolume = NULL;
static TDensity hdensity = NULL;
static TEnthalpy henthalpy = NULL;
static TEntropy hentropy = NULL;
static TIntEnergy hintenergy = NULL;
static TVaporQual hvaporqual = NULL;
static TLiquidCmp hliquidcmp = NULL;
static TVaporCmp hvaporcmp = NULL;
static THeatCapV hheatcapv = NULL;
static THeatCapP hheatcapp = NULL;
static TSoundSpeed hsoundspeed = NULL;
static TAlpha halpha = NULL;
static TBeta hbeta = NULL;
static TChi hchi = NULL;
static TFi hfi = NULL;
static TKsi hksi = NULL;
static TPsi hpsi = NULL;
static TZeta hzeta = NULL;
static TTheta htheta = NULL;
static TKappa hkappa = NULL;
static TGamma hgamma = NULL;
static TViscosity hviscosity = NULL;
static TThermCond hthermcond = NULL;
static TSurfTens hsurftens = NULL;
static TMmol hmmol = NULL;
static TTcrit htcrit = NULL;
static TPcrit hpcrit = NULL;
static TTmin htmin = NULL;
static TTmax htmax = NULL;
static TAllInfo hallinfo = NULL;
static TSetUnits hsetunits = NULL;
static TSetRefState hsetrefstate = NULL;
static TSetRefState_NonDim hsetrefstate_nondim = NULL;
static TzFlow_vu hzflow_vu = NULL;
static TSaturationLine hSaturationLine = NULL;
static TAllTransProps hAllTransProps = NULL;

#define FP_symbol_sublibtest "fluidprop_main_mp_sublibtest_"   // Is dit handig? Het hoeft niet per se.
#define FP_symbol_createobject "fluidprop_base_mp_createobject_"
#define FP_symbol_releaseobjects "fluidprop_base_mp_releaseobjects_"
#define FP_symbol_getversion "fluidprop_main_mp_fp_getversion_"
#define FP_symbol_setfluid "fluidprop_base_mp_setfluid_"
#define FP_symbol_getfluid "fluidprop_base_mp_getfluid_"
#define FP_symbol_settable "fluidprop_base_mp_settable_"
#define FP_symbol_settable_new "fluidprop_base_mp_settable_new_"
#define FP_symbol_usetable "fluidprop_base_mp_usetable_"
#define FP_symbol_getfluidnames "fluidprop_base_mp_getfluidnames_"
#define FP_symbol_allpropssat "fluidprop_main_mp_allpropssat_"
#define FP_symbol_allprops "fluidprop_main_mp_allprops_"
#define FP_symbol_allpropssu2 "fluidprop_main_mp_allpropssu2_"
#define FP_symbol_allpropsjoe "fluidprop_main_mp_allpropsjoe_"
#define FP_symbol_liquidcmp "fluidprop_main_mp_liquidcmp_"
#define FP_symbol_vaporcmp "fluidprop_main_mp_vaporcmp_"
#define FP_symbol_allinfo "fluidprop_main_mp_allinfo_"
#define FP_symbol_setunits "fluidprop_units_mp_setunits_"
#define FP_symbol_setrefstate "fluidprop_refstate_mp_setrefstate_"
#define FP_symbol_setrefstate_nondim "fluidprop_refstate_mp_setrefstate_nondim_"
#define FP_symbol_zflow_vu "fluidprop_main_mp_zflow_vu_"
#define FP_symbol_solve "fluidprop_main_mp_solve_"
#define FP_symbol_pressure "fluidprop_main_mp_pressure_"
#define FP_symbol_temperature "fluidprop_main_mp_temperature_"
#define FP_symbol_specvolume "fluidprop_main_mp_specvolume_"
#define FP_symbol_density "fluidprop_main_mp_density_"
#define FP_symbol_enthalpy "fluidprop_main_mp_enthalpy_"
#define FP_symbol_entropy "fluidprop_main_mp_entropy_"
#define FP_symbol_intenergy "fluidprop_main_mp_intenergy_"
#define FP_symbol_vaporqual "fluidprop_main_mp_vaporqual_"
#define FP_symbol_heatcapv "fluidprop_main_mp_heatcapv_"
#define FP_symbol_heatcapp "fluidprop_main_mp_heatcapp_"
#define FP_symbol_soundspeed "fluidprop_main_mp_soundspeed_"
#define FP_symbol_alpha "fluidprop_main_mp_alpha_"
#define FP_symbol_beta "fluidprop_main_mp_beta_"
#define FP_symbol_chi "fluidprop_main_mp_chi_"
#define FP_symbol_fi "fluidprop_main_mp_fi_"
#define FP_symbol_ksi "fluidprop_main_mp_ksi_"
#define FP_symbol_psi "fluidprop_main_mp_psi_"
#define FP_symbol_zeta "fluidprop_main_mp_zeta_"
#define FP_symbol_theta "fluidprop_main_mp_theta_"
#define FP_symbol_kappa "fluidprop_main_mp_kappa_"
#define FP_symbol_gamma "fluidprop_main_mp_gamma_"
#define FP_symbol_viscosity "fluidprop_main_mp_viscosity_"
#define FP_symbol_thermcond "fluidprop_main_mp_thermcond_"
#define FP_symbol_surftens "fluidprop_main_mp_surftens_"
#define FP_symbol_mmol "fluidprop_main_mp_mmol_"
#define FP_symbol_tcrit "fluidprop_main_mp_tcrit_"
#define FP_symbol_pcrit "fluidprop_main_mp_pcrit_"
#define FP_symbol_tmin "fluidprop_main_mp_tmin_"
#define FP_symbol_tmax "fluidprop_main_mp_tmax_"
#define FP_symbol_saturationline "fluidprop_main_mp_saturationline_"
#define FP_symbol_alltransprops "fluidprop_main_mp_alltransprops_"
#define FP_symbol_if97_getversion "if97version_mp_getversion_"
#define FP_symbol_vthermo_getversion "vthermoversion_mp_getversion_"
#define FP_symbol_tpsi_getversion "tpsiversion_mp_getversion_"
#define FP_symbol_freestanmix_getversion "freestanmixversion_mp_getversion_"
#define FP_symbol_refprop_getversion "refpropversion_mp_getversion_"
#define FP_symbol_gasmix_getversion "gasmixversion_mp_getversion_"
#define FP_symbol_stanmix_getversion "stanmixversion_mp_getversion_"
#define FP_symbol_pcp_saft_getversion "pcp_saftversion_mp_getversion_"
#define FP_symbol_scalinglaws_getversion "scalinglawsversion_mp_getversion_"
#define FP_symbol_luteos_getversion "luteos_version_mp_getversion_"

//====================================================//
//  Procedures for object initialization and cleanup  //
//====================================================//

// Is library loaded?
BOOL fluidprop_isinit(void) 
{
   return (hLibrary != NULL);
}

// Load library
BOOL init_fluidprop(void)
{
   if (!hLibrary)
   {
#if defined __linux__
      hLibrary = dlopen( DEBUGPREFIX "libFluidProp.so", RTLD_LAZY);
#elif defined __APPLE__
      hLibrary = dlopen( DEBUGPREFIX "libfreeStanMix.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libGasMix.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libIF97.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libLiBrH2O.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libLuTEoS.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libPCP-SAFT.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libRefProp.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libScalingLaws.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libStanMix.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libTPSI.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libvThermo.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libFluidProp.dylib", RTLD_LAZY);
#endif
   }
   return (hLibrary != NULL);

/*   if (hLibrary)
      strcpy(ErrorMsg, NO_ERRORS);
   else
      strcpy(ErrorMsg, dlerror());

   LOAD_DL(hGasMix, "GasMix")
   LOAD_DL(hIF97, "IF97")
   LOAD_DL(hfreeStanMix, "freeStanMix")
   LOAD_DL(hTPSI, "TPSI")

#ifdef WITH_REFPROP
   LOAD_DL(hRefProp, "RefProp")
#endif

#ifdef WITH_VTHERMO
   LOAD_DL(hvThermo, "vThermo")
#endif

#ifdef WITH_STANMIX
   LOAD_DL(hStanMix, "StanMix")
#endif

#ifdef WITH_PCP_SAFT
   LOAD_DL(hPCP_SAFT, "PCP-SAFT")
#endif

#ifdef WITH_SCALINGLAWS
   LOAD_DL(hScalingLaws, "ScalingLaws")
#endif

#ifdef WITH_LUTEOS
   LOAD_DL(hLuTEoS, "LuTEoS")
#endif

   // find the fluidprop location
   readconfig("/etc/fluidprop.conf");
   readconfig("~/.fluidproprc");
   readenv();

   if (!strlen(FLUIDPROP) || !strlen(FLUIDPROP_DATA))
      strcpy(ErrorMsg, "FLUIDPROP variables not set, please edit your configuration");

   return (hLibrary != NULL) &&

      (hTPSI != NULL) &&
      (hfreeStanMix != NULL) &&
      (hGasMix != NULL) &&
      (hIF97 != NULL) &&

#ifdef WITH_REFPROP
      (hRefProp != NULL) &&
#endif

#ifdef WITH_VTHERMO
      (hvThermo != NULL) &&
#endif

#ifdef WITH_STANMIX
      (hStanMix != NULL) &&
#endif

#ifdef WITH_PCP_SAFT
      (hPCP_SAFT != NULL) &&
#endif

#ifdef WITH_SCALINGLAWS
      (hScalingLaws != NULL) &&
#endif

#ifdef WITH_LUTEOS
      (hLuTEoS != NULL) &&
#endif

      strlen(FLUIDPROP) &&
      strlen(FLUIDPROP_DATA); */
}

// Unload library 
void uninit_fluidprop()
{
   /*if (!hreleaseobjects)
   {
      *(void**)(&hreleaseobjects) = dlsym(hLibrary, FP_symbol_releaseobjects);
   }

   if (hreleaseobjects)
   {
      (*hreleaseobjects)();
   }*/

   if (hLibrary)
   {
      dlclose(hLibrary);
      hLibrary = NULL;
   }

   /*if (hTPSI)
   {
      dlclose(hTPSI);
      hTPSI = NULL;
   }
   if (hfreeStanMix)
   {
      dlclose(hfreeStanMix);
      hfreeStanMix = NULL;
   }
   if (hGasMix)
   {
      dlclose(hGasMix);
      hGasMix = NULL;
   }
   if (hIF97)
   {
      dlclose(hIF97);
      hIF97 = NULL;
   }
#ifdef WITH_REFPROP
   if (hRefProp)
   {
      dlclose(hRefProp);
      hRefProp = NULL;
   }
#endif
#ifdef WITH_VTHERMO
   if (hvThermo)
   {
      dlclose(hvThermo); 
      hvThermo = NULL; 
   }
#endif
#ifdef WITH_STANMIX
   if (hStanMix)
   {
      dlclose(hStanMix);
      hStanMix = NULL;
   }
#endif
#ifdef WITH_PCP_SAFT
   if (hPCP_SAFT)
   {
      dlclose(hPCP_SAFT);
      hPCP_SAFT = NULL;
   }
#endif
#ifdef WITH_SCALINGLAWS
   if (hScalingLaws)
   {
      dlclose(hScalingLaws);
      hScalingLaws = NULL;
   }
#endif
#ifdef WITH_LUTEOS
   if (hLuTEoS)
   {
      dlclose(hLuTEoS);
      hLuTEoS = NULL;
   }
#endif*/
   }

//==============================================================//
//  Procedures for the calculation of thermodynamic properties  //
//==============================================================//

void sublibtest()
   {
    if (!hSubLibTest)
    {
        *(void**)(&hSubLibTest) = dlsym(hLibrary, "fluidprop_main_mp_sublibtest_");
    }
    if (hSubLibTest)
    {
        (*hSubLibTest) ();
    }
    else
    {
        DEBUGWARNING(fname)
   }
}

// Create object
void fluidprop_createobject( const char* ModelName)
{
  SIZE_TYPE len_ModelName = strlen(ModelName);

  if (!hcreateobject)
    *(void**)(&hcreateobject) = dlsym(hLibrary, FP_symbol_createobject);
  if(hcreateobject)
    (*hcreateobject) ( ModelName, ErrorMsg, len_ModelName, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}

// Returns version array (4 longs)
BOOL fluidprop_getversion(const char* ModelName, int *version)
{
    SIZE_TYPE len_ModelName = strlen(ModelName);
    
   int32_t version32[4];
   int i;

    if (!ModelName || !version)
   {
      strcpy (ErrorMsg, "fluidprop: getversion: null pointer passed");
      return FALSE;
   }

   for (i=0; i<4; ++i)
      version[i] = 0;

    if (!hgetversion)
   {
        *(void**)(&hgetversion) = dlsym(hLibrary, "fluidprop_main_mp_fp_getversion_");
   }
   if (hgetversion)
   {
        (*hgetversion)(ModelName, version32, len_ModelName);

      for (i=0; i<4; ++i)
         version[i] = version32[i];

      strcpy (ErrorMsg, "No errors");
      return TRUE;
   }

   if (dlerror())
     strcpy (ErrorMsg, dlerror());

   return FALSE;
}

char* fluidprop_geterror()
{
  ErrorMsg[len_ErrorMsg] = '\0';
  trimF2C( ErrorMsg );
  return ErrorMsg;
}

const char* fluidprop_getinstallpath()
{
    //return FLUIDPROP;
    return "/Application/FluidProp/";
}

const char* fluidprop_getdatapath()
{
    //return FLUIDPROP_DATA;
    return "/Application/FluidProp/fluids/";
}

// functions calculating thermodynamic properties
void fluidprop_setfluid( const char*  ModelName, int  nComp, const char* Comp, int len_Comp, const double* Conc)
{
  int32_t i32_nComp = nComp; 

  SIZE_TYPE len_ModelName = strlen(ModelName);
  SIZE_TYPE s_len_Comp = len_Comp;

  char localComp[1024];
  int i;

  memcpy(localComp, Comp, nComp*len_Comp);

  for (i=0;i<nComp; ++i)
  {
    untrimC2F(localComp + i * len_Comp, len_Comp);
  }

  if (!hsetfluid)
    {
        //printf("cFPapi, fluidprop.c: finding symbol for setfluid...\n");
    *(void**)(&hsetfluid) = dlsym(hLibrary, FP_symbol_setfluid);
    }

  if(hsetfluid)
    {
        //printf("cFPapi, fluidprop.c: calling hsetfluid...with %s...\n", FP_symbol_setfluid);
    (*hsetfluid)(ModelName, &i32_nComp, localComp, Conc, ErrorMsg, len_ModelName, s_len_Comp, len_ErrorMsg);
        if (!strcmp(fluidprop_geterror(), "Same fluid")) strcpy(ErrorMsg,"No errors");
        //printf("cfpapi, fluidprop.c: hsetfluid finished... %s\n", ErrorMsg);
    }
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_getfluid( char*  ModelName, int len_ModelName, int*  nComp, char*  Comp, int len_Comp, double* Conc)
{
  SIZE_TYPE s_len_ModelName = len_ModelName;
  SIZE_TYPE s_len_Comp = len_Comp;

  int32_t i32_nComp = 0;
  int i;

  if (!hgetfluid)
    *(void**)(&hgetfluid) = dlsym(hLibrary, FP_symbol_getfluid);
  if(hgetfluid)
    (*hgetfluid) 
    ( ModelName, &i32_nComp, Comp, Conc, s_len_ModelName, s_len_Comp);

  for (i=i32_nComp; i; --i)
  {
    Comp[i * len_Comp - 1] = '\0';
    trimF2C( Comp + (i-1) * len_Comp );
  }

  *nComp = (int)i32_nComp;
  ModelName[len_ModelName-1] = '\0';
  trimF2C (ModelName);
}

void fluidprop_settable( const char* TableName, const char* TableType, const char* InterpolMethod, const char* MeshName, 
			 const char* SearchAlg, int phase)
{
  SIZE_TYPE len_TableName      = strlen(TableName);
  SIZE_TYPE len_TableType      = strlen(TableType);
  SIZE_TYPE len_InterpolMethod = strlen(InterpolMethod);
  SIZE_TYPE len_MeshName       = strlen(MeshName);
  SIZE_TYPE len_SearchAlg      = strlen(SearchAlg);

  int32_t i32_phase = phase;

  if (!hsettable)
    *(void**)(&hsettable) = dlsym(hLibrary, FP_symbol_settable);

  if(hsettable)
    (*hsettable) (TableName, TableType, InterpolMethod, MeshName, SearchAlg, &i32_phase, ErrorMsg, 
                  len_TableName, len_TableType, len_InterpolMethod, len_MeshName, len_SearchAlg, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_settable_new( const char* TableName, const char* InputSpec, double* Input1_min, 
                             double* Input1_max, double* Input2_min, double* Input2_max, 
                             double* MeshSpacing, char* TableType, char* IntPolMethod, char* SearchAlg) 
{
  SIZE_TYPE len_TableName      = strlen(TableName);
  SIZE_TYPE len_InputSpec      = strlen(InputSpec); 
  SIZE_TYPE len_TableType      = strlen(TableType);
  SIZE_TYPE len_IntPolMethod   = strlen(IntPolMethod);
  SIZE_TYPE len_SearchAlg      = strlen(SearchAlg);

  if (!hsettable_new)
    *(void**)(&hsettable_new) = dlsym(hLibrary, FP_symbol_settable_new);

  if(hsettable_new)
    (*hsettable_new) (TableName, InputSpec, Input1_min, Input1_max, Input2_min, Input2_max, 
                      MeshSpacing, TableType, IntPolMethod, SearchAlg, ErrorMsg, len_TableName, 
                      len_InputSpec, len_TableType, len_IntPolMethod, len_SearchAlg, len_ErrorMsg);
else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_usetable( const char* TableName)
{
  SIZE_TYPE len_TableName = strlen(TableName);

  if (!husetable)
    *(void**)(&husetable) = dlsym(hLibrary, FP_symbol_usetable);

  if(husetable)
    (*husetable) (TableName, ErrorMsg, len_TableName, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_getfluidnames( const char*  LongShort, const char*  ModelName, int*  nFluids, char*  FluidNames, int len_FluidNames)
{
    int32_t i32_nFluids = 0;
  SIZE_TYPE s_len_FluidNames = len_FluidNames;
  SIZE_TYPE len_LongShort = strlen(LongShort);
  SIZE_TYPE len_ModelName = strlen(ModelName);
  int i;

    //printf("Arrived in fluidprop_getfluidnames for %s, requiring %s fluid names \n", ModelName, LongShort);
    
  if (!hgetfluidnames)
    {
        //printf("Looking for GetFluidNames method in FluidProp\n");
    *(void**)(&hgetfluidnames) = dlsym(hLibrary, FP_symbol_getfluidnames);
    }
  if(hgetfluidnames)
    {
        //printf("GetFluidNames method found in FluidProp... now trying to use it...\n");
    (*hgetfluidnames) 
    (  LongShort,  ModelName, &i32_nFluids, FluidNames, ErrorMsg, len_LongShort, len_ModelName, s_len_FluidNames, len_ErrorMsg);
        //printf("GetFluidNames finished!\n");
    }
  else
  {
    strcpy(ErrorMsg, dlerror());
  }

  *nFluids = (int)i32_nFluids;
  for (i=i32_nFluids; i; --i)
  {
    FluidNames[i * len_FluidNames - 1] = '\0';
    trimF2C( FluidNames + (i-1) * len_FluidNames );
  }
}

void fluidprop_allpropssat( const char*  InputSpec, double Input1, double Input2, struct fluidstate_t* OutputState, double* x, double* y)
{
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hallpropssat)
    *(void**)(&hallpropssat) = dlsym(hLibrary, FP_symbol_allpropssat);
  if(hallpropssat)
    (*hallpropssat) 
    (  InputSpec, &Input1, &Input2, &OutputState->P, &OutputState->T, 
        &OutputState->v, &OutputState->d, &OutputState->h, &OutputState->s, &OutputState->u, &OutputState->q,     
        x, y, &OutputState->cv, &OutputState->cp, &OutputState->c, &OutputState->alpha, &OutputState->beta, 
        &OutputState->chi, &OutputState->fi, &OutputState->ksi, &OutputState->psi, &OutputState->zeta, &OutputState->theta,     
        &OutputState->kappa, &OutputState->gamma, &OutputState->eta, &OutputState->lambda, &OutputState->d_liq,   
        &OutputState->d_vap, &OutputState->h_liq, &OutputState->h_vap, &OutputState->T_sat, 
        &OutputState->dd_liq_dP, &OutputState->dd_vap_dP, &OutputState->dh_liq_dP,    
        &OutputState->dh_vap_dP, &OutputState->dT_sat_dP, ErrorMsg, len_InputSpec, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_allprops( const char*  InputSpec, double Input1, double Input2, struct fluidstate_t* OutputState, double* x, double* y)
{
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hallprops)
    *(void**)(&hallprops) = dlsym(hLibrary, FP_symbol_allprops);
  if(hallprops)
    (*hallprops) 
    (  InputSpec, &Input1, &Input2, &OutputState->P, &OutputState->T, 
        &OutputState->v, &OutputState->d, &OutputState->h, &OutputState->s, &OutputState->u, &OutputState->q,     
        x, y, &OutputState->cv, &OutputState->cp, &OutputState->c, &OutputState->alpha, &OutputState->beta, 
        &OutputState->chi, &OutputState->fi, &OutputState->ksi, &OutputState->psi, &OutputState->zeta, &OutputState->theta, 
        &OutputState->kappa, &OutputState->gamma, &OutputState->eta, &OutputState->lambda, ErrorMsg, len_InputSpec, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}


void fluidprop_allprops_su2( const char*  InputSpec, double Input1, double Input2, struct fluidstate_su2_t* OutputState)
{
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hallpropssu2)
    *(void**)(&hallpropssu2) = dlsym(hLibrary, FP_symbol_allpropssu2);
  if(hallpropssu2)
    (*hallpropssu2) 
    (  InputSpec, &Input1, &Input2, &OutputState->P, &OutputState->T, &OutputState->d, &OutputState->s, &OutputState->u,     
       &OutputState->cv, &OutputState->c2, &OutputState->alpha, &OutputState->beta, ErrorMsg, len_InputSpec, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}


void fluidprop_allpropsjoe( const char*  InputSpec, double Input1, double Input2, struct fluidstatejoe_t* OutputState)
{
    SIZE_TYPE len_InputSpec = strlen(InputSpec);
    
    if (!hallpropsjoe)
    *(void**)(&hallpropsjoe) = dlsym(hLibrary, FP_symbol_allpropsjoe);
    if(hallpropsjoe)
    (*hallpropsjoe)
    (  InputSpec, &Input1, &Input2, &OutputState->P, &OutputState->T,
     &OutputState->v, &OutputState->d, &OutputState->h, &OutputState->s, &OutputState->u,
     &OutputState->cv, &OutputState->cp, &OutputState->c, &OutputState->alpha, &OutputState->beta,
     &OutputState->zeta, &OutputState->eta, &OutputState->lambda, &OutputState->dlam_dT, &OutputState->dlam_drho, &OutputState->deta_dT, &OutputState->deta_drho, ErrorMsg, len_InputSpec, len_ErrorMsg);
    else
    strcpy(ErrorMsg, dlerror());
}


void fluidprop_liquidcmp( const char*  InputSpec, double Input1, double Input2, double* Output)
{
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hliquidcmp)
    *(void**)(&hliquidcmp) = dlsym(hLibrary, FP_symbol_liquidcmp);
  if(hliquidcmp)
    (*hliquidcmp) 
    (  InputSpec, &Input1, &Input2, Output, ErrorMsg, len_InputSpec, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_vaporcmp( const char*  InputSpec, double Input1, double Input2, double* Output)
{
  SIZE_TYPE len_InputSpec = strlen(InputSpec);

  if (!hvaporcmp)
    *(void**)(&hvaporcmp) = dlsym(hLibrary, FP_symbol_vaporcmp);
  if(hvaporcmp)
    (*hvaporcmp)
    (  InputSpec, &Input1, &Input2, Output, ErrorMsg, len_InputSpec, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_allinfo( double* M_mol, double* T_crit, double* P_crit, double* T_min, double* T_max)
{
  
  if (!hallinfo)
    *(void**)(&hallinfo) = dlsym(hLibrary, FP_symbol_allinfo);
  if(hallinfo)
    (*hallinfo) 
    ( M_mol, T_crit, P_crit, T_min, T_max, ErrorMsg, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
  

}

void fluidprop_setunits( const char*  UnitSet, const char*  MassOrMole, const char*  Properties, const char*  Units )
{
  SIZE_TYPE len_UnitSet = strlen(UnitSet);
  SIZE_TYPE len_MassOrMole = strlen(MassOrMole);
  SIZE_TYPE len_Properties = strlen(Properties);
  SIZE_TYPE len_Units = strlen(Units);
  
  if (!hsetunits)
    *(void**)(&hsetunits) = dlsym(hLibrary, FP_symbol_setunits);
  if(hsetunits)
    (*hsetunits) 
    (  UnitSet,  MassOrMole,  Properties,  Units, ErrorMsg , len_UnitSet, len_MassOrMole, len_Properties, len_Units, len_ErrorMsg );
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_setrefstate(double T_ref, double P_ref)
{

   if (!hsetrefstate)
      *(void**)(&hsetrefstate) = dlsym(hLibrary, FP_symbol_setrefstate);
   if (hsetrefstate)
      (*hsetrefstate)
      (&T_ref, &P_ref, ErrorMsg, len_ErrorMsg);
   else
      strcpy(ErrorMsg, dlerror());
}

void fluidprop_setrefstate_nondim(double T_ref, double P_ref, double v_ref)
{

   if (!hsetrefstate_nondim)
      *(void**)(&hsetrefstate_nondim) = dlsym(hLibrary, FP_symbol_setrefstate_nondim);
   if (hsetrefstate_nondim)
      (*hsetrefstate_nondim)
      (&T_ref, &P_ref, &v_ref, ErrorMsg, len_ErrorMsg);
   else
      strcpy(ErrorMsg, dlerror());
}

void fluidprop_zflow_vu(struct fluidstate_t* OutputState)
{
  
  if (!hzflow_vu)
    *(void**)(&hzflow_vu) = dlsym(hLibrary, FP_symbol_zflow_vu);
  if(hzflow_vu)
    (*hzflow_vu) 
    (&OutputState->P, &OutputState->T, &OutputState->v, 
        &OutputState->h, &OutputState->s, &OutputState->u, 
        &OutputState->alpha, &OutputState->beta, &OutputState->chi, 
        &OutputState->fi, &OutputState->zeta, &OutputState->gamma, ErrorMsg, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
  

}

double fluidprop_solve( const char*  FuncSpec, double FuncVal, const char*  InputSpec, int TargetProp, double FixedVal, double MinVal, double MaxVal)
{
  double retval;
  int32_t i32_TargetProp = TargetProp;
  SIZE_TYPE len_FuncSpec = strlen(FuncSpec);
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hsolve)
    *(void**)(&hsolve) = dlsym(hLibrary, FP_symbol_solve);
  if(hsolve)
    retval = (*hsolve) 
    (  FuncSpec, &FuncVal, InputSpec, &i32_TargetProp, &FixedVal, &MinVal, &MaxVal, ErrorMsg, len_FuncSpec, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_pressure( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hpressure)
    *(void**)(&hpressure) = dlsym(hLibrary, FP_symbol_pressure);
  if(hpressure)
    retval = (*hpressure) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_temperature( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!htemperature)
    *(void**)(&htemperature) = dlsym(hLibrary, FP_symbol_temperature);
  if(htemperature)
    retval = (*htemperature) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_specvolume( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hspecvolume)
    *(void**)(&hspecvolume) = dlsym(hLibrary, FP_symbol_specvolume);
  if(hspecvolume)
    retval = (*hspecvolume) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_density( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hdensity)
    *(void**)(&hdensity) = dlsym(hLibrary, FP_symbol_density);
  if(hdensity)
    retval = (*hdensity) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_enthalpy( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!henthalpy)
    *(void**)(&henthalpy) = dlsym(hLibrary, FP_symbol_enthalpy);
  if(henthalpy)
    retval = (*henthalpy) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_entropy( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hentropy)
    *(void**)(&hentropy) = dlsym(hLibrary, FP_symbol_entropy);
  if(hentropy)
    retval = (*hentropy) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_intenergy( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hintenergy)
    *(void**)(&hintenergy) = dlsym(hLibrary, FP_symbol_intenergy);
  if(hintenergy)
    retval = (*hintenergy) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_vaporqual( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hvaporqual)
    *(void**)(&hvaporqual) = dlsym(hLibrary, FP_symbol_vaporqual);
  if(hvaporqual)
    retval = (*hvaporqual) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_heatcapv( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hheatcapv)
    *(void**)(&hheatcapv) = dlsym(hLibrary, FP_symbol_heatcapv);
  if(hheatcapv)
    retval = (*hheatcapv) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_heatcapp( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hheatcapp)
    *(void**)(&hheatcapp) = dlsym(hLibrary, FP_symbol_heatcapp);
  if(hheatcapp)
    retval = (*hheatcapp) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_soundspeed( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hsoundspeed)
    *(void**)(&hsoundspeed) = dlsym(hLibrary, FP_symbol_soundspeed);
  if(hsoundspeed)
    retval = (*hsoundspeed) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_alpha( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!halpha)
    *(void**)(&halpha) = dlsym(hLibrary, FP_symbol_alpha);
  if(halpha)
    retval = (*halpha) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_beta( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hbeta)
    *(void**)(&hbeta) = dlsym(hLibrary, FP_symbol_beta);
  if(hbeta)
    retval = (*hbeta) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_chi( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hchi)
    *(void**)(&hchi) = dlsym(hLibrary, FP_symbol_chi);
  if(hchi)
    retval = (*hchi) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_fi( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hfi)
    *(void**)(&hfi) = dlsym(hLibrary, FP_symbol_fi);
  if(hfi)
    retval = (*hfi) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_ksi( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hksi)
    *(void**)(&hksi) = dlsym(hLibrary, FP_symbol_ksi);
  if(hksi)
    retval = (*hksi) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_psi( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hpsi)
    *(void**)(&hpsi) = dlsym(hLibrary, FP_symbol_psi);
  if(hpsi)
    retval = (*hpsi) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_zeta( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hzeta)
    *(void**)(&hzeta) = dlsym(hLibrary, FP_symbol_zeta);
  if(hzeta)
    retval = (*hzeta) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_theta( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!htheta)
    *(void**)(&htheta) = dlsym(hLibrary, FP_symbol_theta);
  if(htheta)
    retval = (*htheta) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_kappa( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hkappa)
    *(void**)(&hkappa) = dlsym(hLibrary, FP_symbol_kappa);
  if(hkappa)
    retval = (*hkappa) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_gamma( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hgamma)
    *(void**)(&hgamma) = dlsym(hLibrary, FP_symbol_gamma);
  if(hgamma)
    retval = (*hgamma) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_viscosity( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hviscosity)
    *(void**)(&hviscosity) = dlsym(hLibrary, FP_symbol_viscosity);
  if(hviscosity)
    retval = (*hviscosity) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);


  return retval;
}
double fluidprop_thermcond( const char*  InputSpec, double Input1, double Input2)
{
  double retval;
  SIZE_TYPE len_InputSpec = strlen(InputSpec);
  
  if (!hthermcond)
    *(void**)(&hthermcond) = dlsym(hLibrary, FP_symbol_thermcond);
  if(hthermcond)
    retval = (*hthermcond) 
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);

  return retval;
}
double fluidprop_surftens( const char*  InputSpec, double Input1, double Input2)
{
    double retval;
    SIZE_TYPE len_InputSpec = strlen(InputSpec);
    
    if (!hsurftens)
    *(void**)(&hsurftens) = dlsym(hLibrary, FP_symbol_surftens);
    if(hsurftens)
    retval = (*hsurftens)
    (  InputSpec, &Input1, &Input2, ErrorMsg, len_InputSpec, len_ErrorMsg);
    
    return retval;
}
double fluidprop_mmol()
{
  double retval;
  if (!hmmol)
    *(void**)(&hmmol) = dlsym(hLibrary, FP_symbol_mmol);
  if(hmmol)
    retval = (*hmmol) 
    (ErrorMsg, len_ErrorMsg);

  return retval;
}
double fluidprop_tcrit()
{
  double retval;
  if (!htcrit)
    *(void**)(&htcrit) = dlsym(hLibrary, FP_symbol_tcrit);
  if(htcrit)
    retval = (*htcrit) 
    (ErrorMsg, len_ErrorMsg);
  return retval;
}
double fluidprop_pcrit()
{
  double retval;
  if (!hpcrit)
    *(void**)(&hpcrit) = dlsym(hLibrary, FP_symbol_pcrit);
  if(hpcrit)
    retval = (*hpcrit) 
    (ErrorMsg, len_ErrorMsg);
  return retval;
}
double fluidprop_tmin()
{
  double retval;
  if (!htmin)
    *(void**)(&htmin) = dlsym(hLibrary, FP_symbol_tmin);
  if(htmin)
    retval = (*htmin) 
    (ErrorMsg, len_ErrorMsg);
  return retval;
}
double fluidprop_tmax()
{
  double retval;
  if (!htmax)
    *(void**)(&htmax) = dlsym(hLibrary, FP_symbol_tmax);
  if(htmax)
    retval = (*htmax) 
    (ErrorMsg, len_ErrorMsg);
  return retval;
}

void fluidprop_saturationline( int* nPnts, double* T, double* P, double* x, double* y)
{
    if (!hSaturationLine)
    *(void**)(&hSaturationLine) = dlsym(hLibrary, FP_symbol_saturationline);
    if(hSaturationLine)
    (*hSaturationLine)
    (  nPnts, T, P, x, y, ErrorMsg, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}

void fluidprop_alltransprops( const char* InputSpec, double Input1, double Input2, double* eta, double* eta_T, 
                              double* eta_rho, double* lambda, double* lambda_T, double* lambda_rho, double* sigma)
{
  SIZE_TYPE len_InputSpec = strlen(InputSpec);

  if (!hAllTransProps)
    *(void**)(&hAllTransProps) = dlsym(hLibrary, FP_symbol_alltransprops);
  if(hAllTransProps)
    (*hAllTransProps)
    (  InputSpec, &Input1, &Input2, eta, eta_T, eta_rho, lambda, lambda_T, lambda_rho, sigma, ErrorMsg, len_InputSpec, len_ErrorMsg);
  else
    strcpy(ErrorMsg, dlerror());
}



