!========================================================================================!
!                                                                                        !
!  FluidProp X      Teus van der Stelt    Copyright (c) 2004-2018. All Rights Reserved.  !
!  ===========                            Asimptote/iTernergy                            !
!                                                                                        !
!========================================================================================!

   module FluidProp

      ! This module is used as an interface to the FluidProp shared library .  
      
      implicit none

   contains
     
      !==================================================!
      ! Procedures for object initialization and cleanup !
      !==================================================!

      !----------------------------------------------------------------------------------!
         function FluidProp_IsInit()
      !----------------------------------------------------------------------------------!
 
            logical FluidProp_IsInit
            integer(4) FluidProp_IsInit_C

            FluidProp_IsInit = (0 /= FluidProp_IsInit_C())

         end function FluidProp_IsInit

      !----------------------------------------------------------------------------------!
         subroutine Init_FluidProp     
      !----------------------------------------------------------------------------------!

            integer(4) success, Init_FluidProp_C

            success = Init_FluidProp_C()
            if (success == 0) then
               write (*,'(" Unable to load FluidProp library; Aborting")') 
               call exit()
            endif

         end subroutine Init_FluidProp

      !--------------------------------------------------------------------------------------!
         subroutine CreateObject( ModelName, ErrorMsg)     
      !--------------------------------------------------------------------------------------!

            ! Declaration of arguments
            character(len=*), intent(in)  :: ModelName
            character(len=*), intent(out) :: ErrorMsg

            call FluidProp_CreateObject_C( ModelName, ErrorMsg)

         end subroutine CreateObject

      !----------------------------------------------------------------------------------!
         subroutine ReleaseObjects
      !----------------------------------------------------------------------------------!

            call UnInit_FluidProp_C

         end subroutine ReleaseObjects


      !----------------------------------------------------------------------------------!
         subroutine SubLibTest()
      !----------------------------------------------------------------------------------!

            call SubLibTest_C()

         end subroutine SubLibTest


      !=========================================!
      ! Procedure to return version information !
      !=========================================!
      
      !----------------------------------------------------------------------------------!
         function GetVersion(ModelName) result(version)
      !----------------------------------------------------------------------------------!
 
            implicit none

            character(len=*), intent(in) :: ModelName
            integer :: version(4) 

            call FluidProp_GetVersion_C( ModelName, version) 

         end function GetVersion

 
      !====================================!
      ! Procedures to define working fluid !
      !====================================!

      !----------------------------------------------------------------------------------!
         subroutine SetFluid( ModelName, nComp, Comp, Conc, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none

            ! Declaration of arguments
            character(len=*), intent(in)  :: ModelName
            integer(4)      , intent(in)  :: nComp
            character(len=*), intent(in)  :: Comp(nComp)
            real(8)         , intent(in)  :: Conc(nComp)
            character(len=*), intent(out) :: ErrorMsg

            call FluidProp_SetFluid_C( ModelName, nComp, Comp, Conc, ErrorMsg)
            if ( ErrorMsg == 'Same fluid') ErrorMsg = 'No errors'

         end subroutine SetFluid


      !----------------------------------------------------------------------------------!
         subroutine GetFluid( ModelName, nComp, Comp, Conc)
      !----------------------------------------------------------------------------------!

            implicit none

            ! Declaration of arguments
            character(len=*), intent(out) :: ModelName
            integer(4)      , intent(out) :: nComp
            character(len=*), intent(out) :: Comp(:)
            real(8)         , intent(out) :: Conc(:)

            call FluidProp_GetFluid_C( ModelName, nComp, Comp, Conc) 

         end subroutine GetFluid


      !--------------------------------------------------------------------------------------!
         subroutine SetTable( TableName, TableType, InterpolMethod, MeshName,           &
                              SearchAlg, phase, ErrorMsg)
      !--------------------------------------------------------------------------------------!

            character(len=*), intent(in)   :: TableName, TableType, InterpolMethod,     &
                                              MeshName, SearchAlg
            integer         , intent(in)   :: phase
            character(len=*), intent(out)  :: ErrorMsg

            ! Set table parameters, going into the COM-server
            call FluidProp_SetTable_C( TableName, TableType, InterpolMethod, MeshName,  &
                                       SearchAlg, phase, ErrorMsg)
                       
         end subroutine SetTable
 
      
      !--------------------------------------------------------------------------------------!
         subroutine SetTable_New( TableName, InputSpec, Input1_min, Input1_max,             &
                              Input2_min, Input2_max, MeshSpacing, TableType,           &
                              IntpolMethod, SearchAlg, ErrorMsg)
      !--------------------------------------------------------------------------------------!

            character(len=*), intent(in)   :: TableName, InputSpec, TableType,          &
                                              IntpolMethod, SearchAlg
            real(8)         , intent(in)   :: Input1_min, Input1_max, Input2_min,       &
                                              Input2_max, MeshSpacing
            character(len=*), intent(out)  :: ErrorMsg

            ! Set table parameters, going into the COM-server
            call FluidProp_SetTable_New_C( TableName, InputSpec, Input1_min, Input1_max,    &
                                       Input2_min, Input2_max, MeshSpacing, TableType,  &
                                       IntpolMethod, SearchAlg, ErrorMsg)
                       
         end subroutine SetTable_New
 
      
      !--------------------------------------------------------------------------------------!
         subroutine UseTable( TableName, ErrorMsg)
      !--------------------------------------------------------------------------------------!

            character(len=*), intent(in)   :: TableName
            character(len=*), intent(out)  :: ErrorMsg

            call FluidProp_UseTable_C( TableName, ErrorMsg)
                       
         end subroutine UseTable
       

      !==========================================!
      ! Procedure returning available components !
      !==========================================!

      !----------------------------------------------------------------------------------!
         subroutine GetFluidNames( LongShort, ModelName, nFluids, FluidNames, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none

            ! Declaration of arguments
            character(len=*)  , intent(in)  :: LongShort
            character(len=*)  , intent(in)  :: ModelName
            integer(4)        , intent(out) :: nFluids
            character(len=256), intent(out) :: FluidNames(:)
            character(len=*)  , intent(out) :: ErrorMsg


            call FluidProp_GetFluidNames_C( LongShort, ModelName,   &
                                                          nFluids, FluidNames, ErrorMsg) 

         end subroutine GetFluidNames


      !==================================================!
      ! Procedures to calculate thermodynamic properties !
      !==================================================!

      !----------------------------------------------------------------------------------!
         subroutine AllPropsSat( InputSpec, Input1, Input2,                              &
                                 P, T, v, d, h, s, u, q, x, y, cv, cp, c,                &
                                 alpha, beta, chi, fi, ksi, psi, zeta, theta, kappa,     &
                                 gamma, eta, lambda, d_liq, d_vap, h_liq, h_vap, T_sat,  &
                                 dd_liq_dP, dd_vap_dP, dh_liq_dP, dh_vap_dP, dT_sat_dP,  &
                                 ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg
            real(8)         , intent(out) :: P, T, v, d, h, s, u, q, x(20), y(20), cv,   &
                                             cp, c, alpha, beta, chi, fi, ksi, psi,      &
                                             zeta, theta, kappa, gamma, eta, lambda,     &
                                             d_liq, d_vap, h_liq, h_vap, T_sat,          &
                                             dd_liq_dP, dd_vap_dP,dh_liq_dP, dh_vap_dP,  &
                                             dT_sat_dP

            ! Init values (preventing compiler warnings)
            x = 0.d0
            y = 0.d0

            ! Calculate thermodynamic properties
            call FluidProp_AllPropsSat_C( InputSpec, Input1,   &
                                                        Input2, P, T, v, d, h, s, u, q,     &
                                                        x, y, cv, cp, c, alpha, beta, &
                                                        chi, fi, ksi, psi, zeta, theta,     &
                                                        kappa, gamma, eta, lambda, d_liq,   &
                                                        d_vap, h_liq, h_vap, T_sat,         &
                                                        dd_liq_dP, dd_vap_dP, dh_liq_dP,    &
                                                        dh_vap_dP, dT_sat_dP, ErrorMsg)
         end subroutine AllPropsSat

      !----------------------------------------------------------------------------------!
         subroutine AllProps( InputSpec, Input1, Input2,                                 &
                              P, T, v, d, h, s, u, q, x, y, cv, cp, c,                   &
                              alpha, beta, chi, fi, ksi, psi, zeta, theta, kappa, gamma, &
                              eta, lambda, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg
            real(8)         , intent(out) :: P, T, v, d, h, s, u, q, x(20), y(20), cv,   &
                                             cp, c, alpha, beta, chi, fi, ksi, psi,      &
                                             zeta, theta, kappa, gamma, eta, lambda

            ! Init values (preventing compiler warnings)
            x = 0.d0
            y = 0.d0

            ! Calculate thermodynamic properties
            call FluidProp_AllProps_C( InputSpec, Input1,   &
                                                     Input2, P, T, v, d, h, s, u, q,     &
                                                     x, y, cv, cp, c, alpha, beta, &
                                                     chi, fi, ksi, psi, zeta, theta,     &
                                                     kappa, gamma, eta, lambda, ErrorMsg)
         end subroutine AllProps

      !----------------------------------------------------------------------------------!
         subroutine AllPropsSU2( InputSpec, Input1, Input2,                              &
                                 P, T, d, s, u, cv, c2, alpha, beta, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg
            real(8)         , intent(out) :: P, T, d, s, u, cv, c2, alpha, beta

            ! Calculate thermodynamic properties
            call FluidProp_AllPropsSU2_C( InputSpec, Input1, Input2,                     &
                                          P, T, d, s, u, cv, c2, alpha, beta, ErrorMsg)
         end subroutine AllPropsSU2

      !----------------------------------------------------------------------------------!
         subroutine AllPropsJoe( InputSpec, Input1, Input2,                              &
                                 P, T, v, d, h, s, u, cv, cp, c, alpha, beta, zeta, eta, &
                                 lambda, dlam_dT, dlam_drho, deta_dT, deta_drho, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg
            real(8)         , intent(out) :: P, T, v, d, h, s, u, cv,   &
                                             cp, c, alpha, beta, zeta, eta, &
                                             lambda, dlam_dT, dlam_drho, deta_dT, deta_drho

            ! Calculate thermodynamic properties
            call FluidProp_AllPropsJoe_C( InputSpec, Input1, Input2,                        &
                                          P, T, v, d, h, s, u, cv, cp, c, alpha, beta,      &
                                          zeta, eta, lambda, dlam_dT, dlam_drho, deta_dT,   &
                                          deta_drho, ErrorMsg)
         end subroutine AllPropsJoe

  
      !--------------------------------------------------------------------------------------!
         subroutine AllTransProps( InputSpec, Input1, Input2, eta, eta_T, eta_rho,       &
                                   lambda, lambda_T, lambda_rho, sigma, ErrorMsg)
      !--------------------------------------------------------------------------------------!

            ! Calculation of all of FluidProp's properties [user specified units].
            ! --------------------------------------------------------------------

            implicit none
 
            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg
            real(8)         , intent(out) :: eta, eta_T, eta_rho, lambda, lambda_T,      &
                                             lambda_rho, sigma

            call FluidProp_AllTransProps_C( InputSpec, Input1, Input2, eta, eta_T,       &
                                            eta_rho, lambda, lambda_T, lambda_rho,       &
                                            sigma, ErrorMsg)

         end subroutine AllTransProps

      
    !----------------------------------------------------------------------------------!
         function Solve( FuncSpec, FuncVal, InputSpec, TargetProp, FixedVal, MinVal,     &
                         MaxVal, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Solve

            ! Declaration of arguments
            character(len=*), intent(in)  :: FuncSpec, InputSpec
            integer(4)      , intent(in)  :: TargetProp
            real(8)         , intent(in)  :: FuncVal, FixedVal, MinVal, MaxVal
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            ! Calculate thermodynamic properties
            call FluidProp_Solve_C( FuncSpec, FuncVal,      &
                                                  InputSpec, TargetProp, FixedVal,       &
                                                  MinVal, MaxVal, Output, ErrorMsg)                                                     
            ! Return Output as function value 
            Solve = Output 

         end function Solve

      !----------------------------------------------------------------------------------!
         function Pressure( InputSpec, Input1, Input2, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Pressure

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Pressure_C( InputSpec, Input1,      &
                                                     Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Pressure = Output 

         end function Pressure

      !----------------------------------------------------------------------------------!
         function Temperature( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Temperature

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Temperature_C( InputSpec, Input1,   &
                                                        Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Temperature = Output 
     
         end function Temperature

      !----------------------------------------------------------------------------------!
         function SpecVolume( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: SpecVolume

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_SpecVolume_C( InputSpec, Input1,    &
                                                       Input2, Output, ErrorMsg)

            ! Return Output as function value 
            SpecVolume = Output 
     
         end function SpecVolume

      !----------------------------------------------------------------------------------!
         function Density( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Density

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Density_C( InputSpec, Input1,       &
                                                    Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Density = Output 

         end function Density

      !----------------------------------------------------------------------------------!
         function Enthalpy( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Enthalpy

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Enthalpy_C( InputSpec, Input1,      &
                                                     Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Enthalpy = Output 

         end function Enthalpy

      !----------------------------------------------------------------------------------!
         function Entropy( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Entropy

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Entropy_C( InputSpec, Input1,       &
                                                    Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Entropy = Output 

         end function Entropy

      !----------------------------------------------------------------------------------!
         function IntEnergy( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: IntEnergy

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_IntEnergy_C( InputSpec, Input1,     &
                                                      Input2, Output, ErrorMsg)

            ! Return Output as function value 
            IntEnergy = Output 

         end function IntEnergy

      !----------------------------------------------------------------------------------!
         function GibbsEnergy( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: GibbsEnergy

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_GibbsEnergy_C( InputSpec, Input1,     &
                                                      Input2, Output, ErrorMsg)

            ! Return Output as function value 
            GibbsEnergy = Output 

         end function GibbsEnergy

      !----------------------------------------------------------------------------------!
         function VaporQual( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: VaporQual

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_VaporQual_C( InputSpec, Input1,     &
                                                      Input2, Output, ErrorMsg)

            ! Return Output as function value 
            VaporQual = Output 

         end function VaporQual

      !----------------------------------------------------------------------------------!
         subroutine LiquidCmp( InputSpec, Input1, Input2, Output, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            real(8)         , intent(out) :: Output(*)
            character(len=*), intent(out) :: ErrorMsg


            ! calculate phase composition
            call FluidProp_LiquidCmp_C( InputSpec, Input1,     &
                                                      Input2, Output, ErrorMsg)

         end subroutine LiquidCmp

      !----------------------------------------------------------------------------------!
         subroutine VaporCmp( InputSpec, Input1, Input2, Output, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            real(8)         , intent(out) :: Output(*)
            character(len=*), intent(out) :: ErrorMsg

            !  calculate phase composition
            call FluidProp_VaporCmp_C( InputSpec, Input1,      &
                                                     Input2, Output, ErrorMsg)

         end subroutine VaporCmp

      !----------------------------------------------------------------------------------!
         subroutine FugaCoef( InputSpec, Input1, Input2, Output, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            real(8)         , intent(out) :: Output(*)
            character(len=*), intent(out) :: ErrorMsg

            !  calculate phase composition
            call FluidProp_FugaCoef_C( InputSpec, Input1,      &
                                                     Input2, Output, ErrorMsg)

         end subroutine FugaCoef

      !----------------------------------------------------------------------------------!
         function HeatCapV( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: HeatCapV

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_HeatCapV_C( InputSpec, Input1,      &
                                                     Input2, Output, ErrorMsg)

            ! Return Output as function value 
            HeatCapV = Output 

         end function HeatCapV

      !----------------------------------------------------------------------------------!
         function HeatCapP( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: HeatCapP

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_HeatCapP_C( InputSpec, Input1,      &
                                                     Input2, Output, ErrorMsg)

            ! Return Output as function value 
            HeatCapP = Output 

         end function HeatCapP

      !----------------------------------------------------------------------------------!
         function SoundSpeed( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: SoundSpeed

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_SoundSpeed_C( InputSpec, Input1,    &
                                                       Input2, Output, ErrorMsg)

            ! Return Output as function value 
            SoundSpeed = Output 

         end function SoundSpeed

      !----------------------------------------------------------------------------------!
         function Alpha( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Alpha

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Alpha_C( InputSpec, Input1,        &
                                                  Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Alpha = Output 

         end function Alpha

      !----------------------------------------------------------------------------------!
         function Beta( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Beta

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Beta_C( InputSpec, Input1,          &
                                                 Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Beta = Output 

         end function Beta

      !----------------------------------------------------------------------------------!
         function Chi( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Chi

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Chi_C( InputSpec, Input1,           &
                                                Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Chi = Output 

         end function Chi

      !----------------------------------------------------------------------------------!
         function Fi( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Fi

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Fi_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Fi = Output 

         end function Fi

      !----------------------------------------------------------------------------------!
         function Ksi( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Ksi

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Ksi_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Ksi = Output 

         end function Ksi

      !----------------------------------------------------------------------------------!
         function Psi( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Psi

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Psi_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Psi = Output 

         end function Psi

      !----------------------------------------------------------------------------------!
         function Zeta( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Zeta

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Zeta_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Zeta = Output 

         end function Zeta

      !----------------------------------------------------------------------------------!
         function Theta( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Theta

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Theta_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Theta = Output 

         end function Theta

      !----------------------------------------------------------------------------------!
         function Kappa( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Kappa

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Kappa_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Kappa = Output 

         end function Kappa

      !----------------------------------------------------------------------------------!
         function Gamma( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Gamma

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Gamma_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Gamma = Output 

         end function Gamma

      !----------------------------------------------------------------------------------!
         function Viscosity( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Viscosity

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Viscosity_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            Viscosity = Output 

         end function Viscosity

      !----------------------------------------------------------------------------------!
         function ThermCond( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: ThermCond

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_ThermCond_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            ThermCond = Output 

         end function ThermCond


      !----------------------------------------------------------------------------------!
         function SurfTens( InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: SurfTens

            ! Declaration of arguments
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            ! Calculate thermodynamic property
            call FluidProp_SurfTens_C( InputSpec, Input1, Input2, Output, ErrorMsg)

            ! Return Output as function value 
            SurfTens = Output 

         end function SurfTens


      !----------------------------------------------------------------------------------!
         function CalcProp( PropSpec, InputSpec, Input1, Input2, ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: CalcProp

            ! Declaration of arguments
            character(len=*), intent(in)  :: PropSpec, InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            character(len=*), intent(out) :: ErrorMsg

            ! Declaration of local variables
	         real(8) :: Output

            ! Calculate thermodynamic property
            call FluidProp_CalcProp_C( PropSpec, InputSpec, Input1, Input2, Output, &
                                       ErrorMsg)

            ! Return Output as function value 
            CalcProp = Output 

         end function CalcProp

      
      !==========================================!
      ! Utility procedures returning fluid info  !
      !==========================================!

      !----------------------------------------------------------------------------------!
         function Mmol( ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Mmol
            character(len=80) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Mmol_C( Output, ErrorMsg)

            ! Return Output as function value 
            Mmol = Output 

         end function Mmol


      !----------------------------------------------------------------------------------!
         function Tcrit( ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Tcrit
            character(len=80) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Tcrit_C( Output, ErrorMsg)

            ! Return Output as function value 
            Tcrit = Output 

         end function Tcrit


      !----------------------------------------------------------------------------------!
         function Pcrit( ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Pcrit
            character(len=80) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Pcrit_C( Output, ErrorMsg)

            ! Return Output as function value 
            Pcrit = Output 

         end function Pcrit


      !----------------------------------------------------------------------------------!
         function Tmin( ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Tmin
            character(len=80) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Tmin_C( Output, ErrorMsg)

            ! Return Output as function value 
            Tmin = Output 

         end function Tmin


      !----------------------------------------------------------------------------------!
         function Tmax( ErrorMsg)
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Function declaration (= output value)
            real(8) :: Tmax
            character(len=80) :: ErrorMsg

            ! Declaration of local variables
            real(8) :: Output

            !  calculate thermodynamic property
            call FluidProp_Tmax_C( Output, ErrorMsg)

            ! Return Output as function value 
            Tmax = Output 

         end function Tmax


      !----------------------------------------------------------------------------------!
         subroutine AllInfo( M_mol, T_crit, P_crit, T_min, T_max, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8),           intent(in)  :: M_mol, T_crit, P_crit, T_min, T_max
            character(len=80), intent(out) :: ErrorMsg

            !  calculate thermodynamic property
            call FluidProp_AllInfo_C( M_mol, T_crit, P_crit, T_min, T_max, ErrorMsg) 

         end subroutine AllInfo


      !----------------------------------------------------------------------------------!
         subroutine SaturationLine( nPnts, T, P, x, y, ErrorMsg)
      !----------------------------------------------------------------------------------!

            ! Subroutine returning the data of the satuation line.
            ! ----------------------------------------------------

            ! Declaration of arguments
            integer         , intent(out) :: nPnts
            real(8)         , intent(out) :: T(:), P(:), x(:,:), y(:,:)
            character(len=*), intent(out) :: ErrorMsg

            ! Retrieve data in FluidProp unit
            call FluidProp_SaturationLine_C( nPnts, T, P, x, y, ErrorMsg)

         end subroutine SaturationLine

   
      !----------------------------------------------------------------------------------!
         subroutine IsoLine( PropSpec, PropVal, RunSpec, CalcSpec, MinVal, MaxVal,      &
                             nPnts, RunVal, CalcVal, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none

            ! Declaration of arguments
            character(len=*), intent(in)  :: PropSpec
            real(8)         , intent(in)  :: PropVal
            character(len=*), intent(in)  :: RunSpec
            character(len=*), intent(in)  :: CalcSpec
            real(8)         , intent(in)  :: MinVal
            real(8)         , intent(in)  :: MaxVal
            integer         , intent(in)  :: nPnts
            real(8)         , intent(out) :: RunVal(nPnts)
            real(8)         , intent(out) :: CalcVal(nPnts)
            character(len=*), intent(out) :: ErrorMsg

            ! Calculate isoline
            call FluidProp_IsoLine_C( PropSpec, PropVal, RunSpec, CalcSpec, MinVal,     &
                                      MaxVal, nPnts, RunVal, CalcVal, ErrorMsg) 
                                                           
         end subroutine IsoLine


      !======================================================!
      ! Procedures for setting the units and reference state !
      !======================================================!

      !----------------------------------------------------------------------------------!
         subroutine SetUnits( UnitSet, MassOrMole, Properties, Units, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            character(len=*) , intent(in)  :: UnitSet, MassOrMole, Properties, Units
            character(len=80), intent(out) :: ErrorMsg

            !  calculate thermodynamic property
            call FluidProp_SetUnits_C( UnitSet, MassOrMole, Properties, Units, ErrorMsg) 

         end subroutine SetUnits

 
      !----------------------------------------------------------------------------------!
         subroutine SetRefState( T_ref, P_ref, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: T_ref, P_ref
            character(len=80), intent(out) :: ErrorMsg

            ! Set reference state
            call FluidProp_SetRefState_C( T_ref, P_ref, ErrorMsg) 

         end subroutine SetRefState


      !----------------------------------------------------------------------------------!
         subroutine SetRefState_NonDim( T_ref, P_ref, v_ref, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: T_ref, P_ref, v_ref
            character(len=80), intent(out) :: ErrorMsg

            ! Set reference state non-dimensional
            call FluidProp_SetRefState_NonDim_C( T_ref, P_ref, v_ref, ErrorMsg) 

         end subroutine SetRefState_NonDim


      !===============================!
      ! Procedures specific for zFlow !
      !===============================!

      !----------------------------------------------------------------------------------!
         subroutine zFlow_vu( P, T, v, h, s, u, alpha, beta, chi, fi, zeta, gamma, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: v, u
            real(8)          , intent(out) :: P, T, h, s, alpha, beta, chi, fi, zeta,   &
                                              gamma
            character(len=80), intent(out) :: ErrorMsg

            !  calculate thermodynamic property
            call FluidProp_zFlow_vu_C( P, T, v, h, s, u, alpha, beta, chi, fi, zeta,    &
                                       gamma, ErrorMsg) 

         end subroutine zFlow_vu

 
      !----------------------------------------------------------------------------------!
         subroutine CapeOpenDeriv( InputSpec, Input1, Input2, v_der, h_der, s_der,      &
                                   G_der, lnphi_der, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            character(len=*), intent(in)  :: InputSpec
            real(8)         , intent(in)  :: Input1, Input2
            real(8)         , intent(out) :: v_der(:), h_der(:), s_der(:), G_der(:),    &
                                             lnphi_der(:,:)
            character(len=*), intent(out) :: ErrorMsg

            call FluidProp_CapeOpenDeriv_C( InputSpec, Input1, Input2, v_der, h_der,    &
                                            s_der, G_der, lnphi_der, ErrorMsg)

         end subroutine CapeOpenDeriv


      !======================!
      ! Remaining procedures !
      !======================!
      
      !----------------------------------------------------------------------------------!
         subroutine StanMix_Psat_k1( k1, T, Psat, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: k1, T
            real(8)          , intent(out) :: Psat
            character(len=80), intent(out) :: ErrorMsg

            call FluidProp_StanMix_Psat_k1_C( k1, T, Psat, ErrorMsg)

         end subroutine StanMix_Psat_k1


      !----------------------------------------------------------------------------------!
         subroutine StanMix_xy_A_alfa( Aij, Aji, alfa, P, T, x, y, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: Aij, Aji, alfa, P, T
            real(8)          , intent(out) :: x(2), y(2)
            character(len=80), intent(out) :: ErrorMsg

            call FluidProp_StanMix_xy_A_alfa_C( Aij, Aji, alfa, P, T, x, y, ErrorMsg)
                                                
         end subroutine StanMix_xy_A_alfa


      !----------------------------------------------------------------------------------!
         subroutine PCP_SAFT_xy_kij( kij, P, T, x, y, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: kij, P, T
            real(8)          , intent(out) :: x(2), y(2)
            character(len=80), intent(out) :: ErrorMsg

            call FluidProp_PCP_SAFT_xy_kij_C( kij, P, T, x, y, ErrorMsg)

         end subroutine PCP_SAFT_xy_kij


      !----------------------------------------------------------------------------------!
         subroutine PCP_SAFT_P_kij( kij, T, q, P, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: kij, T, q
            real(8)          , intent(out) :: P
            character(len=80), intent(out) :: ErrorMsg

            call FluidProp_PCP_SAFT_P_kij_C( kij, T, q, P, ErrorMsg)

         end subroutine PCP_SAFT_P_kij


      !----------------------------------------------------------------------------------!
         subroutine PCP_SAFT_T_kij( kij, P, q, T, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: kij, P, q
            real(8)          , intent(out) :: T
            character(len=80), intent(out) :: ErrorMsg

            call FluidProp_PCP_SAFT_T_kij_C( kij, P, q, T, ErrorMsg)

         end subroutine PCP_SAFT_T_kij


      !----------------------------------------------------------------------------------!
         subroutine PCP_SAFT_hsxy_mp( P, T, MolPar, h_r, s_r, x, y, ErrorMsg) 
      !----------------------------------------------------------------------------------!

            implicit none
 
            ! Argument declaration
            real(8)          , intent(in)  :: P, T, MolPar(:,:)
            real(8)          , intent(out) :: h_r, s_r, x(:), y(:)
            character(len=80), intent(out) :: ErrorMsg

            ! Get h, s, x and y as function of P, T and the molecular parameters
            call FluidProp_PCP_SAFT_hsxy_mp_C( P, T, MolPar, h_r, s_r, x, y, ErrorMsg)

         end subroutine PCP_SAFT_hsxy_mp
 
end module FluidProp

!================================================================================= EOF ==!

