//============================================================================================!
//                                                                                            !
//  FluidProp      Teus van der Stelt        Copyright (c) 2004-2008. All Rights Reserved.    !
//  =========      Piero Colonna             Energy Technology Section, TU Delft.             !
//                                                                                            !
//                                                                                            !
//                                                                                            !
//  Module name   : FluidProp                                                                 !
//                                                                                            !   
//  Description   : This module loads and provides access to the FLUIDPROP library.           !
//                                                                                            !   
//  Contents      : Procedures for object initialization and cleanup:                         !   
//                  1.  FLUIDPROP_IsInit                                                      !   
//                  2.  Init_FLUIDPROP                                                        !   
//                  3.  UnInit_FLUIDPROP                                                      !   
//                                                                                            !   
//                  Procedures according the FLUIDPROP interface:                             !   
//                  4. SubLibTest                                                             !
//                                                                                            !   
//  Creation date : 21-07-2008, Henk Seubers, VORtech                                         !
//                                                                                            !   
//============================================================================================!

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include "config.h"
#include "shared.h"
#include "fstring.h"
#include "FluidPropC.h"

// Library handle (NULL in case of uninitialized) 
static void *hLibrary = NULL;

// Library function signatures
typedef void (FP_SO_CALL *TSubLibTest)    ();

typedef void (FP_SO_CALL *TGetVersion)    ( char* ModelName, int32_t* version, SIZE_TYPE len_ModelName);

typedef void (FP_SO_CALL *TCreateObject)  ( char* ModelName, char* ErrorMsg,
                                            SIZE_TYPE len_ModelName, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TReleaseObjects)(void);

typedef void (FP_SO_CALL *TSetFluid)      ( char* ModelName, int32_t* nComp, char* Comp, 
                                            double* Conc, char* ErrorMsg,
                                            SIZE_TYPE len_ModelName, SIZE_TYPE len_Comp,
                                            SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TGetFluid)      ( char* ModelName, int32_t* nComp, char* Comp, 
                                            double* Conc, SIZE_TYPE len_ModelName,
                                            SIZE_TYPE len_Comp);

// Function for table setup
typedef void (FP_SO_CALL *TSetTable)      ( char* TableName, char* TableType, char* InterpolMethod, 
				                                char* MeshName, char* SearchAlg, int32_t* phase, 
				                                char* ErrorMsg, SIZE_TYPE len_TableName, 
                                            SIZE_TYPE len_TableType, SIZE_TYPE len_InterpolMethod, 
                                            SIZE_TYPE len_MeshName, SIZE_TYPE len_SearchAlg, 
                                            SIZE_TYPE len_ErrorMsg);
                                            
typedef void (FP_SO_CALL *TSetTable_New)      ( char* TableName, char* InputSpec, double* Input1_min, 
                                            double* Input1_max, double* Input2_min, double* Input2_max, 
                                            double* MeshSpacing, char* TableType, char* IntPolMethod, char* SearchAlg, 
                                            char* ErrorMsg, SIZE_TYPE len_TableName, SIZE_TYPE len_InputSpec, 
                                            SIZE_TYPE len_TableType, SIZE_TYPE len_IntPolMethod,
                                            SIZE_TYPE len_SearchAlg, SIZE_TYPE len_ErrorMsg);

// Function for table use
typedef void (FP_SO_CALL *TUseTable)      ( char* TableName, char* ErrorMsg, 
                                            SIZE_TYPE len_TableName, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TGetFluidNames) ( char* LongShort, char* ModelName, int32_t* nFluids, 
											           char* FluidNames, char* ErrorMsg,
											           SIZE_TYPE len_LongShort, SIZE_TYPE len_ModelName,
                                            SIZE_TYPE len_FluidNames, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllPropsSat)   ( char* InputSpec, double* Input1, double* Input2, 
											double* P, double* T, double* v, double* d, double* h, 
											double* s, double* u, double* q, double* x, double* y, 
                                            double* cv, double* cp, double* c, double* alpha,
                                            double* beta,double* chi, double* fi, double* ksi, 
                                            double* psi, double* zeta, double* theta, double* kappa, 
                                            double* gamma, double* eta, double* lambda, 
                                            double* d_liq, double* d_vap, double* h_liq, 
                                            double* h_vap, double* T_sat, double* dd_liq_dP, 
                                            double* dd_vap_dP, double* dh_liq_dP, double* dh_vap_dP, 
                                            double* dT_sat_dP, char* ErrorMsg,
                                            SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllProps)      ( char* InputSpec, double* Input1, double* Input2, double* P, double* T,
                                            double* v, double* d, double* h, double* s, double* u, double* q,
                                            double* x, double* y, double* cv, double* cp, double* c, double* alpha,
                                            double* beta, double* chi, double* fi, double* ksi, double* psi,
                                            double* zeta, double* theta, double* kappa, double* gamma, double* eta,
                                            double* lambda, char* ErrorMsg, SIZE_TYPE len_InputSpec,
                                            SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllPropsSU2)   ( char* InputSpec, double* Input1, double* Input2, double* P, double* T, double* d, double* s, double* u,      
                                            double* cv, double* c2, double* alpha, double* beta, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllPropsJoe)   ( char* InputSpec, double* Input1, double* Input2,
                                            double* P, double* T, double* v, double* d, double* h, double* s, double* u,      
                                            double* cv, double* cp, double* c, double* alpha, double* beta, double* zeta,      
                                            double* eta, double* lambda, double* dlam_dT, double* dlam_drho, double* deta_dT, double* deta_drho, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllTransProps)(  char* InputSpec, double* Input1, double* Input2, double* eta, double* 
                                            eta_T, double* eta_rho, double* lambda, double* lambda_T, double* 
                                            lambda_rho, double* sigma, char* ErrorMsg, SIZE_TYPE len_InputSpec,
                                            SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSolve)( char* FuncSpec, double* FuncVal,      char* InputSpec, int32_t* TargetProp, double* FixedVal,       
                                                  double* MinVal, double* MaxVal,  char* ErrorMsg, SIZE_TYPE len_FuncSpec, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TPressure)( char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TTemperature)( char* InputSpec, double* Input1,   double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSpecVolume)( char* InputSpec, double* Input1,    double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TDensity)( char* InputSpec, double* Input1,       double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TEnthalpy)( char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TEntropy)( char* InputSpec, double* Input1,       double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TIntEnergy)( char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TGibbsEnergy)( char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TVaporQual)( char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TLiquidCmp)( char* InputSpec, double* Input1,     double* Input2,  double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TVaporCmp)( char* InputSpec, double* Input1,      double* Input2,  double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TFugaCoef) ( char* InputSpec, double* Input1,      double* Input2,  double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *THeatCapV)( char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *THeatCapP)( char* InputSpec, double* Input1,      double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSoundSpeed)( char* InputSpec, double* Input1,    double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TAlpha)( char* InputSpec, double* Input1,        double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TBeta)( char* InputSpec, double* Input1,          double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TChi)( char* InputSpec, double* Input1,           double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TFi)( char* InputSpec, double* Input1,            double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TKsi)( char* InputSpec, double* Input1,           double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TPsi)( char* InputSpec, double* Input1,           double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TZeta)( char* InputSpec, double* Input1,          double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TTheta)( char* InputSpec, double* Input1,         double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TKappa)( char* InputSpec, double* Input1,         double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TGamma)( char* InputSpec, double* Input1,         double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TViscosity)( char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TThermCond)( char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TSurfTens)( char* InputSpec, double* Input1,     double* Input2,  char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TCalcProp)( char* PropSpec, char* InputSpec, double* Input1, double* Input2,  char* ErrorMsg,  SIZE_TYPE len_PropSpec,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef double (FP_SO_CALL *TMmol)(  char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TTcrit)(  char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TPcrit)(  char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TTmin)(  char* ErrorMsg, SIZE_TYPE len_ErrorMsg);
typedef double (FP_SO_CALL *TTmax)(  char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TAllInfo)( double* M_mol, double* T_crit, double* P_crit, double* T_min, double* T_max, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSetUnits)( char* UnitSet, char* MassOrMole, char* Properties, char* Units, char* ErrorMsg, 
                     SIZE_TYPE len_UnitSet, SIZE_TYPE len_MassOrMole, SIZE_TYPE len_Properties, SIZE_TYPE len_Units, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSetRefState)(double* T_ref, double* P_ref, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSetRefState_NonDim)(double* T_ref, double* P_ref, double* v_ref, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TzFlow_vu)( double* P, double* T, double* v, double* h, double* s, double* u, double* alpha, double* beta, double* chi, double* fi, double* zeta, double* gamma,
                                                     char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TSaturationLine) ( int32_t* nPnts, double* T, double* P, double* x, double* y, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TIsoLine) ( char* PropSpec, double* PropVal, char* RunSpec, char* CalcSpec, double* MinVal, double* MaxVal, int32_t* nPnts, 
                                      double* RunVal, double* CalcVal, char* ErrorMsg, SIZE_TYPE len_PropSpec, SIZE_TYPE len_RunSpec, SIZE_TYPE len_CalcSpec, 
                                      SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TCapeOpenDeriv)( char* InputSpec, double* Input1, double* Input2, double* v_der, double* h_der, 
                                                                              double* s_der, double* G_der, double* lnphi_der, char* ErrorMsg,  
                                                                              SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TStanMix_Psat_k1)( double* k1, double* T, double* Psat, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TStanMix_xy_A_alfa)( double* Aij, double* Aji, double* alfa, double* P, double* T, double* x, 
                                                                                     double* y, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TPCP_SAFT_xy_kij)( double* kij, double* P, double* T, double* x, double* y, char* ErrorMsg, 
                                                                                  SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TPCP_SAFT_P_kij)( double* kij, double* T, double* q, double* P, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TPCP_SAFT_T_kij)( double* kij, double* P, double* q, double* T, char* ErrorMsg, SIZE_TYPE len_ErrorMsg);

typedef void (FP_SO_CALL *TPCP_SAFT_hsxy_mp)( double* P, double* T, double* MolPar, double* h_r, double* s_r, double* x, double* y, char* ErrorMsg,
                                                                                    SIZE_TYPE len_ErrorMsg);

// Library function handles
static TSubLibTest hSubLibTest = NULL;
static TGetVersion hGetVersion = NULL;
static TCreateObject hCreateObject = NULL;
static TReleaseObjects hReleaseObjects = NULL;
static TSetFluid hSetFluid = NULL;
static TGetFluid hGetFluid = NULL;
static TGetFluidNames hGetFluidNames = NULL;
static TSetTable hSetTable = NULL;
static TSetTable_New hSetTable_New = NULL;
static TUseTable hUseTable = NULL;
static TAllPropsSat hAllPropsSat = NULL;
static TAllProps hAllProps = NULL;
static TAllPropsSU2 hAllPropsSU2 = NULL; // SU2 version
static TAllPropsJoe hAllPropsJoe = NULL; // Joe version
static TSolve hSolve = NULL;
static TPressure hPressure = NULL;
static TTemperature hTemperature = NULL;
static TSpecVolume hSpecVolume = NULL;
static TDensity hDensity = NULL;
static TEnthalpy hEnthalpy = NULL;
static TEntropy hEntropy = NULL;
static TIntEnergy hIntEnergy = NULL;
static TGibbsEnergy       hGibbsEnergy       = NULL;
static TVaporQual hVaporQual = NULL;
static TLiquidCmp hLiquidCmp = NULL;
static TVaporCmp hVaporCmp = NULL;
static TFugaCoef          hFugaCoef          = NULL;
static THeatCapV hHeatCapV = NULL;
static THeatCapP hHeatCapP = NULL;
static TSoundSpeed hSoundSpeed = NULL;
static TAlpha hAlpha = NULL;
static TBeta hBeta = NULL;
static TChi hChi = NULL;
static TFi hFi = NULL;
static TKsi hKsi = NULL;
static TPsi hPsi = NULL;
static TZeta hZeta = NULL;
static TTheta hTheta = NULL;
static TKappa hKappa = NULL;
static TGamma hGamma = NULL;
static TViscosity hViscosity = NULL;
static TThermCond hThermCond = NULL;
static TSurfTens          hSurfTens          = NULL;
static TCalcProp          hCalcProp          = NULL;
static TMmol hMmol = NULL;
static TTcrit hTcrit = NULL;
static TPcrit hPcrit = NULL;
static TTmin hTmin = NULL;
static TTmax hTmax = NULL;
static TAllInfo hAllInfo = NULL;
static TSetUnits          hSetUnits          = NULL;     
static TSetRefState       hSetRefState = NULL;
static TSetRefState_NonDim hSetRefState_NonDim = NULL;
static TzFlow_vu          hzFlow_vu          = NULL;
static TSaturationLine hSaturationLine = NULL;
static TIsoLine           hIsoLine           = NULL;
static TAllTransProps     hAllTransProps     = NULL;
static TCapeOpenDeriv     hCapeOpenDeriv     = NULL;
static TStanMix_Psat_k1   hStanMix_Psat_k1   = NULL;
static TStanMix_xy_A_alfa hStanMix_xy_A_alfa = NULL;
static TPCP_SAFT_xy_kij   hPCP_SAFT_xy_kij   = NULL;
static TPCP_SAFT_P_kij    hPCP_SAFT_P_kij    = NULL;
static TPCP_SAFT_T_kij    hPCP_SAFT_T_kij    = NULL;
static TPCP_SAFT_hsxy_mp  hPCP_SAFT_hsxy_mp  = NULL;


// ==================================================!
//  Procedures for object initialization and cleanup !
// ==================================================!

// Is library loaded?
BOOL FP_C_CALL FC_FUNC(fluidprop_isinit_c,FLUIDPROP_ISINIT_C)(void) 
{
   return (hLibrary != NULL);
}

// Load library
BOOL FP_C_CALL FC_FUNC(init_fluidprop_c,INIT_FLUIDPROP_C)(void)
{
   //const SIZE_TYPE len_ErrorMsg = 100;
   //char ErrorMsg [len_ErrorMsg];

   if (!hLibrary)
   {
#if defined __linux__
      //printf("Try to open libfreeStanMix.so....\n");
      hLibrary = dlopen( DEBUGPREFIX "libfreeStanMix.so", RTLD_LAZY);
      //if (hLibrary) 
      //   printf("LibfreeStanMix.so opened!\n");
      //else
      //   printf("Opening LibfreeStanMix.so failed!\n");
      hLibrary = dlopen( DEBUGPREFIX "libGasMix.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libIF97.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libLiBrH2O.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libLuTEoS.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libPCP-SAFT.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libRefProp.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libScalingLaws.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libStanMix.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libTPSI.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libvThermo.so", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libFluidProp.so", RTLD_LAZY);
#elif defined __APPLE__
      hLibrary = dlopen( DEBUGPREFIX "libfreeStanMix.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libGasMix.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libIF97.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libLiBrH2O.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libLuTEoS.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libPCP-SAFT.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libRefProp.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libScalingLaws.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libStanMix.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libTPSI.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libvThermo.dylib", RTLD_LAZY);
      hLibrary = dlopen( DEBUGPREFIX "libFluidProp.dylib", RTLD_LAZY);
#endif
   }
   return (hLibrary != NULL);

/*   FC_FUNC(init_tpsi_c,INIT_TPSI_C)(ErrorMsg, len_ErrorMsg);
   FC_FUNC(init_freestanmix_c,INIT_FREESTANMIX_C)(ErrorMsg, len_ErrorMsg);
   FC_FUNC(init_gasmix_c,INIT_GASMIX_C)(ErrorMsg, len_ErrorMsg);
   FC_FUNC(init_if97_c,INIT_IF97_C)(ErrorMsg, len_ErrorMsg);
#ifdef WITH_REFPROP
   FC_FUNC(init_refprop_c,INIT_REFPROP_C)(ErrorMsg, len_ErrorMsg);
#endif
#ifdef WITH_VTHERMO
   FC_FUNC(init_vthermo_c,INIT_VTHERMO_C)(ErrorMsg, len_ErrorMsg);
#endif
#ifdef WITH_STANMIX
   FC_FUNC(init_stanmix_c,INIT_STANMIX_C)(ErrorMsg, len_ErrorMsg);
#endif
#ifdef WITH_PCP_SAFT
   FC_FUNC(init_pcp_saft_c,INIT_PCP_SAFT_C)(ErrorMsg, len_ErrorMsg);
#endif
#ifdef WITH_SCALINGLAWS
   FC_FUNC(init_scalinglaws_c,INIT_SCALINGLAWS_C)(ErrorMsg, len_ErrorMsg);
#endif
#ifdef WITH_LUTEOS
   //FC_FUNC(init_luteos_c,INIT_LUTEOS_C)(ErrorMsg, len_ErrorMsg);
#endif

   return (hLibrary != NULL) &&
#ifdef WITH_REFPROP
   FC_FUNC(refprop_isinit_c,REFPROP_ISINIT_C)() &&
#endif
#ifdef WITH_VTHERMO
   FC_FUNC(vthermo_isinit_c,VTHERMO_ISINIT_C)() &&
#endif
#ifdef WITH_STANMIX
   FC_FUNC(stanmix_isinit_c,STANMIX_ISINIT_C)() &&
#endif
#ifdef WITH_PCP_SAFT
   FC_FUNC(pcp_saft_isinit_c,PCP_SAFT_ISINIT_C)() &&
#endif
#ifdef WITH_SCALINGLAWS
   FC_FUNC(scalinglaws_isinit_c,SCALINGLAWS_ISINIT_C)() &&
#endif
#ifdef WITH_LUTEOS
   //FC_FUNC(luteos_isinit_c,LUTEOS_ISINIT_C)() &&
#endif
   FC_FUNC(tpsi_isinit_c,TPSI_ISINIT_C)() &&
   FC_FUNC(freestanmix_isinit_c,FREESTANMIX_ISINIT_C)() &&
   FC_FUNC(gasmix_isinit_c,GASMIX_ISINIT_C)() &&
   FC_FUNC(if97_isinit_c,IF97_ISINIT_C)();*/
}

// Create object 
void FP_C_CALL FC_FUNC(fluidprop_createobject_c, FLUIDPROP_CREATEOBJECT_C) ( char* ModelName, char* ErrorMsg, SIZE_TYPE len_ModelName, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hCreateObject, fluidprop_base_mp_createobject_, ( ModelName, ErrorMsg, len_ModelName, len_ErrorMsg))

// Unload library 
void FP_C_CALL FC_FUNC(uninit_fluidprop_c,UNINIT_FLUIDPROP_C)()
{
/*   if (!hReleaseObjects)
   {
      *(void**)(&hReleaseObjects) = dlsym(hLibrary, "fluidprop_base_mp_releaseobjects_");
   }

   if (hReleaseObjects)
   {
      (*hReleaseObjects)();
   }

   FC_FUNC(uninit_tpsi_c,UNINIT_TPSI_C)();
   FC_FUNC(uninit_freestanmix_c,UNINIT_FREESTANMIX_C)();
   FC_FUNC(uninit_gasmix_c,UNINIT_GASMIX_C)();
   FC_FUNC(uninit_if97_c,UNINIT_IF97_C)();
#ifdef WITH_REFPROP
   FC_FUNC(uninit_refprop_c,UNINIT_REFPROP_C)();
#endif
#ifdef WITH_VTHERMO
   FC_FUNC(uninit_vthermo_c,UNINIT_VTHERMO_C)();
#endif
#ifdef WITH_STANMIX
   FC_FUNC(uninit_stanmix_c,UNINIT_STANMIX_C)();
#endif
#ifdef WITH_PCP_SAFT
   FC_FUNC(uninit_pcp_saft_c,UNINIT_PCP_SAFT_C)();
#endif
#ifdef WITH_SCALINGLAWS
   FC_FUNC(uninit_scalinglaws_c,UNINIT_SCALINGLAWS_C)();
#endif
#ifdef WITH_LUTEOS
   //FC_FUNC(uninit_luteos_c,UNINIT_LUTEOS_C)();
#endif*/

   if (hLibrary)
   {
      dlclose(hLibrary);
      hLibrary = NULL;
   }
}

// Returns version array (4 longs)
void FP_C_CALL FC_FUNC(fluidprop_getversion_c,FLUIDPROP_GETVERSION_C) ( char* ModelName, int32_t *version, SIZE_TYPE len_ModelName)
{
   if (!hGetVersion)
   {
        *(void**)(&hGetVersion) = dlsym(hLibrary, "fluidprop_main_mp_fp_getversion_");
   }

   if (hGetVersion)
   {
        (*hGetVersion)( ModelName, version, len_ModelName);
   }
   else
   {
      memset(version, 0, 4 * sizeof(int32_t));
   }
}

//==============================================================//
//  Procedures for the calculation of thermodynamic properties  //
//==============================================================//

void FP_C_CALL FC_FUNC(sublibtest_c, SUBLIBTEST_C)()
{ 
   if (!hSubLibTest)
   { 
      *(void**)(&hSubLibTest) = dlsym(hLibrary, "fluidprop_main_mp_sublibtest_");
   } 
   if (hSubLibTest)
   { 
      (*hSubLibTest) ();
   } 
   else 
   { 
      DEBUGWARNING(fname)
   } 
}
//This is the same:
//FC_SUB(hSubLibTest, fluidprop_main_mp_sublibtest_, ())

// functions calculating thermodynamic properties
void FP_C_CALL FC_FUNC(fluidprop_setfluid_c, FLUIDPROP_SETFLUID_C)( char* ModelName, int32_t* nComp, char* Comp, double* Conc, char* ErrorMsg, SIZE_TYPE len_ModelName, SIZE_TYPE len_Comp, SIZE_TYPE len_ErrorMsg) 
{
    if (!hSetFluid)
    {
        //printf("FluidPropC.c: Looking for SetFluid in library...\n");
        *(void**)(&hSetFluid) = dlsym(hLibrary, "fluidprop_base_mp_setfluid_");
    }
    if (hSetFluid)
    {
        //printf("FluidPropC.c: SetFluid found...calling SetFluid...\n");
        (*hSetFluid) (  ModelName,  nComp,  Comp,  Conc,  ErrorMsg,  len_ModelName,  len_Comp,  len_ErrorMsg);
        //printf("FluidPropC.c: SetFluid finished: %s\n", ErrorMsg);
    }
    else
    {
        strncpyC2F(ErrorMsg, dlerror(), len_ErrorMsg);
        DEBUGWARNING(fname)
        //printf("FluidPropC.c: SetFluid not found: %s\n", ErrorMsg);
    }
}
//This is the same:
//FC_SUB(hSetFluid, fluidprop_base_mp_setfluid_, (  ModelName,  nComp,  Comp,  Conc,  ErrorMsg,  len_ModelName,  len_Comp,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_getfluid_c, FLUIDPROP_GETFLUID_C)( char* ModelName, int32_t* nComp, char* Comp, double* Conc, SIZE_TYPE len_ModelName, SIZE_TYPE len_Comp) 
FC_SUB_NOERR(hGetFluid, fluidprop_base_mp_getfluid_, (  ModelName,  nComp,  Comp,  Conc,  len_ModelName,  len_Comp))

// Function for table setup
void FP_C_CALL FC_FUNC(fluidprop_settable_c, FLUIDPROP_SETTABLE_C)( char* TableName, char* TableType, char* InterpolMethod, 
							        char* MeshName, char* SearchAlg, int32_t* phase, 
							        char* ErrorMsg, SIZE_TYPE len_TableName, 
                                                                SIZE_TYPE len_GenerateMode, SIZE_TYPE len_InterpolMethod, 
                                                                SIZE_TYPE len_StepFunc, SIZE_TYPE len_SearchAlg, 
                                                                SIZE_TYPE len_ErrorMsg)
{
    if (!hSetTable)
    {
        //printf("FluidPropC.c: Looking for SetTable in library...\n");
        *(void**)(&hSetTable) = dlsym(hLibrary, "fluidprop_base_mp_settable_");
    }
    if (hSetTable)
    {
        //printf("FluidPropC.c: SetTable found...calling SetTable...\n");
        (*hSetTable) ( TableName, TableType, InterpolMethod, MeshName, SearchAlg, phase, ErrorMsg, len_TableName, len_GenerateMode, len_InterpolMethod, len_StepFunc, len_SearchAlg, len_ErrorMsg);
        //printf("FluidPropC.c: SetTable finished: %s\n", ErrorMsg);
    }
    else
    {
        strncpyC2F(ErrorMsg, dlerror(), len_ErrorMsg);
        DEBUGWARNING(fname)
        //printf("FluidPropC.c: SetTable not found: %s\n", ErrorMsg);
    }
}

// Function for table setup
void FP_C_CALL FC_FUNC(fluidprop_settable_new_c, FLUIDPROP_SETTABLE_NEW_C)( char* TableName,
                                            char* InputSpec, double* Input1_min, double* Input1_max, 
                                            double* Input2_min, double* Input2_max, 
                                            double* MeshSpacing, char* TableType, char* IntPolMethod, 
                                            char* SearchAlg, char* ErrorMsg, SIZE_TYPE len_TableName, 
                                            SIZE_TYPE len_InputSpec, SIZE_TYPE len_TableType, 
                                            SIZE_TYPE len_IntPolMethod, SIZE_TYPE len_SearchAlg, 
                                            SIZE_TYPE len_ErrorMsg)
{
    if (!hSetTable_New)
    {
        //printf("FluidPropC.c: Looking for SetTable in library...\n");
        *(void**)(&hSetTable_New) = dlsym(hLibrary, "fluidprop_base_mp_settable_new_");
    }
    if (hSetTable_New)
    {
        //printf("FluidPropC.c: SetTable found_New...calling SetTable_New...\n");
        (*hSetTable_New) ( TableName, InputSpec, Input1_min, Input1_max, Input2_min, Input2_max,
                       MeshSpacing, TableType, IntPolMethod, SearchAlg, ErrorMsg, len_TableName, len_InputSpec, 
                       len_TableType, len_IntPolMethod, len_SearchAlg, len_ErrorMsg);
        //printf("FluidPropC.c: SetTable_New finished: %s\n", ErrorMsg);
    }
    else
    {
        strncpyC2F(ErrorMsg, dlerror(), len_ErrorMsg);
        DEBUGWARNING(fname)
        //printf("FluidPropC.c: SetTable_New not found: %s\n", ErrorMsg);
    }
}
//This is the same:
//FC_SUB( hSetTable_New, fluidprop_base_mp_settable_new_, ( TableName, InputSpec, Input1_min, Input1_max, Input2_min, Input2_max, MeshSpacing, TableType, IntPolMethod, SearchAlg, ErrorMsg, len_TableName, len_InputSpec, len_TableType, len_IntPolMethod, len_SearchAlg, len_ErrorMsg))

// Function for table use
void FP_C_CALL FC_FUNC(fluidprop_usetable_c, FLUIDPROP_USETABLE_C)( char* TableName, char* ErrorMsg, SIZE_TYPE len_TableName, SIZE_TYPE len_ErrorMsg)
{
    if (!hUseTable)
    {
        //printf("FluidPropC.c: Looking for UseTable in library...\n");
        *(void**)(&hUseTable) = dlsym(hLibrary, "fluidprop_base_mp_usetable_");
    }
    if (hSetTable)
    {
        //printf("FluidPropC.c: UseTable found...calling UseTable...\n");
        (*hUseTable) ( TableName, ErrorMsg, len_TableName, len_ErrorMsg);
        //printf("FluidPropC.c: UseTable finished: %s\n", ErrorMsg);
    }
    else
    {
        strncpyC2F(ErrorMsg, dlerror(), len_ErrorMsg);
        DEBUGWARNING(fname)
        //printf("FluidPropC.c: UseTable not found: %s\n", ErrorMsg);
    }
}
//FC_SUB( hUseTable, fluidprop_base_mp_usetable_, ( TableName, ErrorMsg, len_TableName, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_getfluidnames_c, FLUIDPROP_GETFLUIDNAMES_C)( char* LongShort, char* ModelName, int32_t* nFluids, char* FluidNames, char* ErrorMsg, SIZE_TYPE len_LongShort, SIZE_TYPE len_ModelName, SIZE_TYPE len_FluidNames, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hGetFluidNames, fluidprop_base_mp_getfluidnames_, (  LongShort,  ModelName,  nFluids,  FluidNames,  ErrorMsg,  len_LongShort,  len_ModelName,  len_FluidNames,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_allpropssat_c, FLUIDPROP_ALLPROPSSAT_C)( char* InputSpec, double* Input1, double* Input2,
                                                                          double* P, double* T, double* v, double* d, double* h, double* s, double* u, double* q,
                                                                          double* x, double* y, double* cv, double* cp, double* c, double* alpha, double* beta,
                                                                          double* chi, double* fi, double* ksi, double* psi, double* zeta, double* theta,
                                                                          double* kappa, double* gamma, double* eta, double* lambda, double* d_liq, double* d_vap,
                                                                          double* h_liq, double* h_vap, double* T_sat, double* dd_liq_dP, double* dd_vap_dP,
                                                                          double* dh_liq_dP, double* dh_vap_dP, double* dT_sat_dP, char* ErrorMsg,
                                                                          SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg)
FC_SUB(hAllPropsSat, fluidprop_main_mp_allpropssat_, (  InputSpec,  Input1,    Input2,  P,  T,  v,  d,  h,  s,  u,  q,      x,  y,  cv,  cp,  c,  alpha,  beta,  chi,  fi,  ksi,  psi,  zeta,  theta,      kappa,  gamma,  eta,  lambda,  d_liq,    d_vap,  h_liq,  h_vap,  T_sat,          dd_liq_dP,  dd_vap_dP,  dh_liq_dP,     dh_vap_dP,  dT_sat_dP,  ErrorMsg,  len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_allprops_c, FLUIDPROP_ALLPROPS_C)      ( char* InputSpec, double* Input1, double* Input2,
                                                                          double* P, double* T, double* v, double* d, double* h, double* s, double* u, double* q,
                                                                          double* x, double* y, double* cv, double* cp, double* c, double* alpha, double* beta,
                                                                          double* chi, double* fi, double* ksi, double* psi, double* zeta, double* theta,
                                                                          double* kappa, double* gamma, double* eta, double* lambda, char* ErrorMsg, SIZE_TYPE len_InputSpec,
                                                                          SIZE_TYPE len_ErrorMsg)
FC_SUB(hAllProps, fluidprop_main_mp_allprops_, (  InputSpec,  Input1,    Input2,  P,  T,  v,  d,  h,  s,  u,  q,      x,  y,  cv,  cp,  c,  alpha,  beta,  chi,  fi,  ksi,  psi,  zeta,  theta,      kappa,  gamma,  eta,  lambda,  ErrorMsg,  len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_allpropssu2_c, FLUIDPROP_ALLPROPSSU2_C)( char* InputSpec, double* Input1,   
      double* Input2, double* P, double* T, double* d, double* s, double* u,      
      double* cv, double* c2, double* alpha, double* beta, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hAllPropsSU2, fluidprop_main_mp_allpropssu2_, (  InputSpec, Input1, Input2,  P,  T,  d, s,  u, cv,  c2,  alpha,  beta, ErrorMsg,  len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_allpropsjoe_c, FLUIDPROP_ALLPROPSJOE_C)( char* InputSpec, double* Input1,   
      double* Input2, double* P, double* T, double* v, double* d, double* h, double* s, double* u,      
      double* cv, double* cp, double* c, double* alpha, double* beta, double* zeta, double* eta, double* lambda, double* dlam_dT, double* dlam_drho, double* deta_dT, double *deta_drho, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hAllPropsJoe, fluidprop_main_mp_allpropsjoe_, (  InputSpec,  Input1,    Input2,  P,  T,  v,  d,  h,  s,  u, cv,  cp,  c,  alpha,  beta, zeta, eta,  lambda, dlam_dT, dlam_drho, deta_dT, deta_drho, ErrorMsg,  len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_alltransprops_c, FLUIDPROP_ALLTRANSPROPS_C)( char* InputSpec, double* Input1, double* Input2, double* eta, double* eta_T, double* eta_rho, double* lambda, double* lambda_T, double* lambda_rho, double* sigma, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg)
FC_SUB(hAllTransProps, fluidprop_main_mp_alltransprops_, ( InputSpec, Input1, Input2, eta, eta_T, eta_rho, lambda, lambda_T, lambda_rho, sigma, ErrorMsg, len_InputSpec, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_solve_c, FLUIDPROP_SOLVE_C)
( char* FuncSpec, double* FuncVal, char* InputSpec, int32_t* TargetProp, double* FixedVal,       
  double* MinVal, double* MaxVal, double* Output, char* ErrorMsg, SIZE_TYPE len_FuncSpec, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hSolve, fluidprop_main_mp_solve_, *Output, (  FuncSpec,  FuncVal,       InputSpec,  TargetProp,  FixedVal,        MinVal,  MaxVal,  ErrorMsg,  len_FuncSpec,  len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_pressure_c, FLUIDPROP_PRESSURE_C)
( char* InputSpec, double* Input1, double* Input2, double* Output, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hPressure, fluidprop_main_mp_pressure_, *Output, (  InputSpec,  Input1,       Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_temperature_c, FLUIDPROP_TEMPERATURE_C)
( char* InputSpec, double* Input1, double* Input2, double* Output, char* ErrorMsg, SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hTemperature, fluidprop_main_mp_temperature_, *Output, (  InputSpec,  Input1,    Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_specvolume_c, FLUIDPROP_SPECVOLUME_C)( char* InputSpec, double* Input1,    double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hSpecVolume, fluidprop_main_mp_specvolume_, *Output, (  InputSpec,  Input1,     Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_density_c, FLUIDPROP_DENSITY_C)( char* InputSpec, double* Input1,       double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hDensity, fluidprop_main_mp_density_, *Output, (  InputSpec,  Input1,        Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_enthalpy_c, FLUIDPROP_ENTHALPY_C)( char* InputSpec, double* Input1,      double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hEnthalpy, fluidprop_main_mp_enthalpy_, *Output, (  InputSpec,  Input1,       Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_entropy_c, FLUIDPROP_ENTROPY_C)( char* InputSpec, double* Input1,       double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hEntropy, fluidprop_main_mp_entropy_, *Output, (  InputSpec,  Input1,        Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_intenergy_c, FLUIDPROP_INTENERGY_C)( char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hIntEnergy, fluidprop_main_mp_intenergy_, *Output, (  InputSpec,  Input1,      Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_gibbsenergy_c, FLUIDPROP_GIBBSENERGY_C)( char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hGibbsEnergy, fluidprop_main_mp_gibbsenergy_, *Output, (  InputSpec,  Input1,      Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_vaporqual_c, FLUIDPROP_VAPORQUAL_C)( char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hVaporQual, fluidprop_main_mp_vaporqual_, *Output, (  InputSpec,  Input1,      Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_liquidcmp_c, FLUIDPROP_LIQUIDCMP_C)( char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hLiquidCmp, fluidprop_main_mp_liquidcmp_, (  InputSpec,  Input1,      Input2,  Output, ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_vaporcmp_c, FLUIDPROP_VAPORCMP_C)( char* InputSpec, double* Input1,      double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hVaporCmp, fluidprop_main_mp_vaporcmp_, (  InputSpec,  Input1,       Input2,  Output, ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_fugacoef_c, FLUIDPROP_FUGACOEF_C)( char* InputSpec, double* Input1,      double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hFugaCoef, fluidprop_main_mp_fugacoef_, (  InputSpec,  Input1,       Input2,  Output, ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_heatcapv_c, FLUIDPROP_HEATCAPV_C)( char* InputSpec, double* Input1,      double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hHeatCapV, fluidprop_main_mp_heatcapv_, *Output, (  InputSpec,  Input1,       Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_heatcapp_c, FLUIDPROP_HEATCAPP_C)( char* InputSpec, double* Input1,      double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hHeatCapP, fluidprop_main_mp_heatcapp_, *Output, (  InputSpec,  Input1,       Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_soundspeed_c, FLUIDPROP_SOUNDSPEED_C)( char* InputSpec, double* Input1,    double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hSoundSpeed, fluidprop_main_mp_soundspeed_, *Output, (  InputSpec,  Input1,     Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_alpha_c, FLUIDPROP_ALPHA_C)( char* InputSpec, double* Input1,        double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hAlpha, fluidprop_main_mp_alpha_, *Output, (  InputSpec,  Input1,         Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_beta_c, FLUIDPROP_BETA_C)( char* InputSpec, double* Input1,          double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hBeta, fluidprop_main_mp_beta_, *Output, (  InputSpec,  Input1,           Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_chi_c, FLUIDPROP_CHI_C)( char* InputSpec, double* Input1,           double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hChi, fluidprop_main_mp_chi_, *Output, (  InputSpec,  Input1,            Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_fi_c, FLUIDPROP_FI_C)( char* InputSpec, double* Input1,            double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hFi, fluidprop_main_mp_fi_, *Output, (  InputSpec,  Input1,             Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_ksi_c, FLUIDPROP_KSI_C)( char* InputSpec, double* Input1,           double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hKsi, fluidprop_main_mp_ksi_, *Output, (  InputSpec,  Input1,            Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_psi_c, FLUIDPROP_PSI_C)( char* InputSpec, double* Input1,           double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hPsi, fluidprop_main_mp_psi_, *Output, (  InputSpec,  Input1,            Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_zeta_c, FLUIDPROP_ZETA_C)( char* InputSpec, double* Input1,          double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hZeta, fluidprop_main_mp_zeta_, *Output, (  InputSpec,  Input1,           Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_theta_c, FLUIDPROP_THETA_C)( char* InputSpec, double* Input1,         double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hTheta, fluidprop_main_mp_theta_, *Output, (  InputSpec,  Input1,          Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_kappa_c, FLUIDPROP_KAPPA_C)( char* InputSpec, double* Input1,         double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hKappa, fluidprop_main_mp_kappa_, *Output, (  InputSpec,  Input1,          Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_gamma_c, FLUIDPROP_GAMMA_C)( char* InputSpec, double* Input1,         double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hGamma, fluidprop_main_mp_gamma_, *Output, (  InputSpec,  Input1,          Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_viscosity_c, FLUIDPROP_VISCOSITY_C)( char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hViscosity, fluidprop_main_mp_viscosity_, *Output, (  InputSpec,  Input1,      Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_thermcond_c, FLUIDPROP_THERMCOND_C)( char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hThermCond, fluidprop_main_mp_thermcond_, *Output, (  InputSpec,  Input1,      Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_surftens_c, FLUIDPROP_SURFTENS_C)( char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hSurfTens, fluidprop_main_mp_surftens_, *Output, (  InputSpec,  Input1,      Input2,  ErrorMsg,   len_InputSpec,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_calcprop_c, FLUIDPROP_CALCPROP_C)( char* PropSpec, char* InputSpec, double* Input1,     double* Input2, double* Output, char* ErrorMsg,  SIZE_TYPE len_PropSpec,  SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hCalcProp, fluidprop_main_mp_calcprop_, *Output, (  PropSpec, InputSpec,  Input1, Input2,  ErrorMsg, len_PropSpec, len_InputSpec, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_mmol_c, FLUIDPROP_MMOL_C)( double* Output, char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hMmol, fluidprop_main_mp_mmol_, *Output, (  ErrorMsg,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_tcrit_c, FLUIDPROP_TCRIT_C)( double* Output, char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hTcrit, fluidprop_main_mp_tcrit_, *Output, (  ErrorMsg,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_pcrit_c, FLUIDPROP_PCRIT_C)( double* Output, char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hPcrit, fluidprop_main_mp_pcrit_, *Output, (  ErrorMsg,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_tmin_c, FLUIDPROP_TMIN_C)( double* Output, char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hTmin, fluidprop_main_mp_tmin_, *Output, (  ErrorMsg,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_tmax_c, FLUIDPROP_TMAX_C)( double* Output, char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_FUN(hTmax, fluidprop_main_mp_tmax_, *Output, (  ErrorMsg,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_allinfo_c, FLUIDPROP_ALLINFO_C)( double* M_mol, double* T_crit, double* P_crit, double* T_min, double* T_max, char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hAllInfo, fluidprop_main_mp_allinfo_, (  M_mol,  T_crit,  P_crit,  T_min,  T_max,  ErrorMsg,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_setunits_c, FLUIDPROP_SETUNITS_C)( char* UnitSet, char* MassOrMole, char* Properties, char* Units, char* ErrorMsg, SIZE_TYPE len_UnitSet, SIZE_TYPE len_MassOrMole, SIZE_TYPE len_Properties, SIZE_TYPE len_Units, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hSetUnits, fluidprop_units_mp_setunits_, (  UnitSet,  MassOrMole,  Properties,  Units,  ErrorMsg,  len_UnitSet,  len_MassOrMole,  len_Properties,  len_Units,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_setrefstate_c, FLUIDPROP_SETREFSTATE_C)(double* T_ref, double* P_ref, char* ErrorMsg, SIZE_TYPE len_ErrorMsg)
FC_SUB(hSetRefState, fluidprop_refstate_mp_setrefstate_, (T_ref, P_ref, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_setrefstate_nondim_c, FLUIDPROP_SETREFSTATE_NONDIM_C)(double* T_ref, double* P_ref, double* v_ref, char* ErrorMsg, SIZE_TYPE len_ErrorMsg)
FC_SUB(hSetRefState_NonDim, fluidprop_refstate_mp_setrefstate_nondim_, (T_ref, P_ref, v_ref, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_zflow_vu_c, FLUIDPROP_ZFLOW_VU_C)( double* P, double* T, double* v, double* h, double* s, double* u,
      double* alpha, double* beta, double* chi, double* fi, double* zeta, double* gamma,      char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_SUB(hzFlow_vu, fluidprop_main_mp_zflow_vu_, (  P,  T,  v,  h,  s,  u,  alpha,  beta,  chi,  fi,  zeta,  gamma,       ErrorMsg,  len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_saturationline_c, FLUIDPROP_SATURATIONLINE_C) ( int32_t* nPnts, double* T, double* P, double* x, double* y, char* ErrorMsg, SIZE_TYPE len_ErrorMsg) 
FC_SUB_NOERR(hSaturationLine, fluidprop_main_mp_saturationline_, ( nPnts, T, P, x, y, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_isoline_c, FLUIDPROP_ISOLINE_C) ( char* PropSpec, double* PropVal, char* RunSpec, char* CalcSpec, double* MinVal, double* MaxVal, int32_t* nPnts, 
                                               double* RunVal, double* CalcVal, char* ErrorMsg, SIZE_TYPE len_PropSpec, SIZE_TYPE len_RunSpec, SIZE_TYPE len_CalcSpec, 
                                               SIZE_TYPE len_ErrorMsg)
FC_SUB(hIsoLine, fluidprop_main_mp_isoline_, ( PropSpec, PropVal, RunSpec, CalcSpec, MinVal, MaxVal, nPnts, RunVal, CalcVal, ErrorMsg, len_PropSpec, len_RunSpec, len_CalcSpec, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_capeopenderiv_c, FLUIDPROP_CAPEOPENDERIV_C)( char* InputSpec, double* Input1, double* Input2, double* v_der, double* h_der, 
                                                                              double* s_der, double* G_der, double* lnphi_der, char* ErrorMsg,  
                                                                              SIZE_TYPE len_InputSpec, SIZE_TYPE len_ErrorMsg)
FC_SUB(hCapeOpenDeriv, fluidprop_main_mp_capeopenderiv_, ( InputSpec, Input1, Input2, v_der, h_der, s_der, G_der, lnphi_der, ErrorMsg, len_InputSpec, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_stanmix_psat_k1_c, FLUIDPROP_STANMIX_PSAT_K1_C)( double* k1, double* T, double* Psat, char* ErrorMsg, SIZE_TYPE len_ErrorMsg)
FC_SUB(hStanMix_Psat_k1, fluidprop_main_mp_stanmix_psat_k1_, ( k1, T, Psat, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_stanmix_xy_a_alfa_c, FLUIDPROP_STANMIX_XY_A_ALFA_C)( double* Aij, double* Aji, double* alfa, double* P, double* T, double* x, 
                                                                                     double* y, char* ErrorMsg, SIZE_TYPE len_ErrorMsg)
FC_SUB(hStanMix_xy_A_alfa, fluidprop_main_mp_stanmix_xy_a_alfa_, ( Aij, Aji, alfa, P, T, x, y, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_pcp_saft_xy_kij_c, FLUIDPROP_PCP_SAFT_XY_KIJ_C)( double* kij, double* P, double* T, double* x, double* y, char* ErrorMsg, 
                                                                                  SIZE_TYPE len_ErrorMsg)
FC_SUB(hPCP_SAFT_xy_kij, fluidprop_main_mp_pcp_saft_xy_kij_, ( kij, P, T, x, y, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_pcp_saft_p_kij_c, FLUIDPROP_PCP_SAFT_P_KIJ_C)( double* kij, double* T, double* q, double* P, char* ErrorMsg, SIZE_TYPE len_ErrorMsg)
FC_SUB(hPCP_SAFT_P_kij, fluidprop_main_mp_pcp_saft_p_kij_, ( kij, T, q, P, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_pcp_saft_t_kij_c, FLUIDPROP_PCP_SAFT_T_KIJ_C)( double* kij, double* P, double* q, double* T, char* ErrorMsg, SIZE_TYPE len_ErrorMsg)
FC_SUB(hPCP_SAFT_T_kij, fluidprop_main_mp_pcp_saft_t_kij_, ( kij, P, q, T, ErrorMsg, len_ErrorMsg))

void FP_C_CALL FC_FUNC(fluidprop_pcp_saft_hsxy_mp_c, FLUIDPROP_PCP_SAFT_HSXY_MP_C)( double* P, double* T, double* MolPar, double* h_r, double* s_r, double* x, double* y, char* ErrorMsg,
                                                                                    SIZE_TYPE len_ErrorMsg)
FC_SUB(hPCP_SAFT_hsxy_mp, fluidprop_main_mp_pcp_saft_hsxy_mp_, ( P, T, MolPar, h_r, s_r, x, y, ErrorMsg, len_ErrorMsg))
