#---------------------------------------------------------------------------------------------------#
#                                                                                                   #                                             
#  Created on November, 2018                                                                      #
#                                                                                                   #                                             
#  Python example for FluidProp on MS-Windows, Linux, and macOS.                                    #
#                                                                                                   #                                             
#  This example uses FluidProp.py instead of calling the FluidProp functions directly.              #
#                                                                                                   #                                             
#  @authors: Teus van der Stelt, Asimptote BV,                                                      #
#            Heeswijk-Dinther, The Netherlands,                                                     #
#            Teus.vanderStelt@Asimptote.com                                                         #
#                                                                                                   #                                             
#---------------------------------------------------------------------------------------------------#

from __future__ import print_function

from FPPython import *

# Before you can use FluidProp in Windows you must create and initialize a FluidProp
# object with the Init_FluidProp call. 

Init_FluidProp()

# Before a property can be calculated, first a child object for the desired 
# thermophysical library must be created. This can be done by using the CreateObject 
# subroutine or by defining the working fluid directly with the SetFluid subroutine. 
# Although subroutine SetFluid can be used for the same purpose too, with the 
# CreateObject subroutine you can easily create a list of available libraries in your client.

version = FluidProp_Version("FluidProp")

if GetErrorMsg() != 'No errors' :
    print ()
    print (GetErrorMsg())
    exit()

else:
    print ()
    print ('Available libraries in FluidProp version ', version[0],'.', version[1],'.', version[2],'.', version[3])
    print ('----------------------------------------------------------------------------')

    # Create an object for the TPSI library
    SubLib = "TPSI"
    CreateObject( SubLib)
    if GetErrorMsg() == 'No errors' :
       version = FluidProp_Version( SubLib)
       print ('TPSI version        ', version[0],'.', version[1],'.', version[2],'.', version[3])
       TPSI_available = True
    else:
       print ('Unable to create TPSI object!')
       TPSI_available = False

    # Create an object for the freeStanMix library
    SubLib = "freeStanMix"
    CreateObject( SubLib)
    if GetErrorMsg() == 'No errors' :
       version = FluidProp_Version( SubLib)
       print ('freeStanMix version ', version[0],'.', version[1],'.', version[2],'.', version[3])
       freeStanMix_available = True
    else:
       print ('Unable to create freeStanMix object!')
       freeStanMix_available = False

    # Create an object for the IF97 library
    SubLib   = "IF97"
    CreateObject( SubLib)
    if GetErrorMsg() == 'No errors' :
       version = FluidProp_Version( SubLib)
       print ('IF97 version        ', version[0],'.', version[1],'.', version[2],'.', version[3])
       IF97_available = True
    else:
       print ('Unable to create IF97 object!')
       IF97_available = False

    # Create an object for the GasMix library
    SubLib   = "GasMix"
    CreateObject( SubLib)
    if GetErrorMsg() == 'No errors' :
       version = FluidProp_Version( SubLib)
       print ('GasMix version      ', version[0],'.', version[1],'.', version[2],'.', version[3])
       GasMix_available = True
    else:
       print ('Unable to create GasMix object!')
       GasMix_available = False

    # Create an object for the RefProp library
    SubLib   = "RefProp"
    CreateObject( SubLib)
    if GetErrorMsg() == 'No errors' :
       version = FluidProp_Version( SubLib)
       print ('RefProp version     ', version[0],'.', version[1],'.', version[2],'.', version[3])
       RefProp_available = True
    else:
       print ('Unable to create RefProp object!')
       RefProp_available = False

    # Create an object for the StanMix library
    SubLib   = "StanMix"
    CreateObject( SubLib)
    if GetErrorMsg() == 'No errors' :
       version = FluidProp_Version( SubLib)
       print ('StanMix version     ', version[0],'.', version[1],'.', version[2],'.', version[3])
       StanMix_available = True
    else:
       print ('Unable to create StanMix object!')
       StanMix_available = False

    # Create an object for the PCP-SAFT library
    SubLib   = "PCP-SAFT"
    CreateObject( SubLib)
    if GetErrorMsg() == 'No errors' :
       version = FluidProp_Version( SubLib)
       print ('PCP-SAFT version    ', version[0],'.', version[1],'.', version[2],'.', version[3])
       PCP_SAFT_available = True
    else:
       print ('Unable to create PCP-SAFT object!')
       PCP_SAFT_available = False


#================================= Examples of FluidProp calculations =============================#

#--------------------------------------------------------------------------------------------------!
#                                               TPSI                                               !
#--------------------------------------------------------------------------------------------------!

if TPSI_available:

    #  After checking if the desired library is available, first the working fluid 
    #  must be defined before a property can be calculated. In this example methane
    #  from the TPSI library is defined as the working fluid. For TPSI there is no
    #  need to specify concentation Conc, because there is always just 1 component.

    SubLib  = "TPSI"
    Comp    = ["methane"]

    SetFluid( SubLib, Comp)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('TPSI, Methane, P = 1 bar, T = 15 degr.C')
       print ('----------------------------------------------------------------------------')
 
       P = 1.
       T = 15.
   
       # To calculate some properties as function of two other independent properties
       # the appropriate function must be called. In the example below the density,
       # specific volume, enthalpy, entropy and vapor quality are calculated as 
       # function of the pressure and temperature. There is a check if the pressure  
       # and temperature function return the same value as entered. 

       P = Pressure( 'PT', P, T)
       print ( 'Pressure      = %7.2f bar      ' % P)
       if GetErrorMsg() != 'No errors': print (GetErrorMsg())

       T = Temperature( 'PT', P, T)
       print ( 'Temperature   = %7.2f C        ' % T)
       if GetErrorMsg() != 'No errors': print (GetErrorMsg())

       d = Density( 'PT', P, T)
       print ( 'Density       = %g kg/m3    ' % d)
       if GetErrorMsg() != 'No errors': print (GetErrorMsg())

       v = SpecVolume( 'PT', P, T)
       print ( 'Spec. volume  = %g m3/kg    ' % v)
       if GetErrorMsg() != 'No errors': print (GetErrorMsg())

       h = Enthalpy( 'PT', P, T)
       print ( 'Enthalpy      = %g kJ/kg    ' % h)
       if GetErrorMsg() != 'No errors': print (GetErrorMsg())

       s = Entropy( 'PT', P, T)
       print ( 'Entropy       = %g kJ/(kg.K)' % s)
       if GetErrorMsg() != 'No errors': print (GetErrorMsg())

       q = VaporQual( 'PT', P, T)
       print ( 'Vapor quality = %g' % q)
       if GetErrorMsg() != 'No errors': print (GetErrorMsg())

    # You can retrieve a list of available fluids in a library by calling subroutine 
    # GetFluidNames. The number of fluids and an array with fluid names is returned. 
    FluidName_spec = "long"
    FluidNames = GetFluidNames( FluidName_spec, SubLib)

    print ()
    print ('Available components in TPSI')
    print ('----------------------------------------------------------------------------')

    if GetErrorMsg() == 'No errors' :
       print ('Number of available fluids = ', len(FluidNames))
       print ('The available fluids are: ')
       i = 0
       while i < len(FluidNames):
          # 2-column output 
          if i == len(FluidNames) - 1:
             print (FluidNames[i].rstrip().ljust(40))
          else:
             print (FluidNames[i].rstrip().ljust(40), FluidNames[i+1].rstrip())
          i += 2

# For the other libraries the method is essentially the same. The only difference is in the way 
# arguments are supplied to the SetFluid subroutine. Furthermore the properties that can be 
# calculated varies for each library, as well as the two the independent properties that can be 
# used as input (see user documentation).     
# In the examples below for each library the use of subroutine SetFluid is explained in detail, 
# together with the calculation of several properties.

#--------------------------------------------------------------------------------------------------!
#                                           freeStanMix                                            !
#--------------------------------------------------------------------------------------------------!

if freeStanMix_available:

    SubLib  = "freeStanMix"

    # You can retrieve a list of available fluids in a library by calling subroutine GetFluidNames. 
    # The number of fluids and an array with fluid names is returned. 
    FluidName_spec = "long"
    FluidNames = GetFluidNames( FluidName_spec, SubLib)

    print ()
    print ('Available components in freeStanMix')
    print ('----------------------------------------------------------------------------')

    if GetErrorMsg() == 'No errors' :
       print ('Number of available fluids = ', len(FluidNames))
       print ('The available fluids are: ')
       i = 0
       while i < len(FluidNames):
          # 2-column output 
          if i == len(FluidNames) - 1:
             print (FluidNames[i].rstrip().ljust(40))
          else:
             print (FluidNames[i].rstrip().ljust(40), FluidNames[i+1].rstrip())
          i += 2

    # For freeStanMix you have three possiblities: you can use pure fluids, predefined mixtures, or you 
    # can use mixtures and assign different mole fractions to concentrations of the components. 
    #
    # In the first example a pure fluid is selected. There is no need to specify a concentration, it is 
    # sufficient to specify the fluidname, in this case ethanol, and the number of components being 1.

    Comp = ["ethanol"]
    SetFluid( SubLib, Comp)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('freeStanMix, ethanol, P = 2 bar, T = 120 degr.C')
       print ('----------------------------------------------------------------------------')
 
       P = 2.0
       T = 120.

       # To calculate some properties as function of two other independent properties
       # the appropriate function must be called. In the example a large number of 
       # properties is calculated. Next to various thermodynamic properties also 
       # properties related to CFD quantities are available. For clarity the writing
       # of the error message has been left out.

       print ('Pressure      = %g bar      ' % Pressure   ('PT', P, T))
       print ('Temperature   = %g degr. C  ' % Temperature('PT', P, T))
       print ('Density       = %g kg/m3    ' % Density    ('PT', P, T))
       print ('Spec. volume  = %g m3/kg    ' % SpecVolume ('PT', P, T))
       print ('Enthalpy      = %g kJ/kg    ' % Enthalpy   ('PT', P, T))
       print ('Entropy       = %g kJ/(kg.K)' % Entropy    ('PT', P, T))
       print ('Vapor quality = %g          ' % VaporQual  ('PT', P, T))
       print ('Int. energy   = %g kJ/kg    ' % IntEnergy  ('PT', P, T))
       print ('Isoc. Heat Cv = %g kJ/(kg.K)' % HeatCapV   ('PT', P, T))
       print ('Isoc. Heat Cp = %g kJ/(kg.K)' % HeatCapP   ('PT', P, T))
       print ('Sound speed   = %g m/s      ' % SoundSpeed ('PT', P, T))
 
       # Note that an additional Add-on is required in order to compute the derivatives. 
       # A license for this Add-on can be purchased from the Asimptote web shop. If the 
       # appropriate license is not available on your system then an error message is 
       # returned by the GetErrorMsg function.           
       Alpha( 'PT', P, T)

       if GetErrorMsg() != 'No errors' :
          print ()
          print (GetErrorMsg())
       else:
          print ('Alpha         = %g m2/s2    ' % Alpha      ('PT', P, T))
          print ('Beta          = %g kg/m3    ' % Beta       ('PT', P, T))
          print ('Chi           = %g Nm4/kg2.K' % Chi        ('PT', P, T))
          print ('Fi            = %g 1/K      ' % Fi         ('PT', P, T))
          print ('Ksi           = %g kg2/N.m4 ' % Ksi        ('PT', P, T))
          print ('Psi           = %g s2/m2    ' % Psi        ('PT', P, T))
          print ('Zeta          = %g kg.K/m3  ' % Zeta       ('PT', P, T))
          print ('Theta         = %g 1/K      ' % Theta      ('PT', P, T))
          print ('Kappa         = %g 1/bar    ' % Kappa      ('PT', P, T))
          print ('Gamma         = %g          ' % Gamma      ('PT', P, T))

    # In the second example a predefined mixture is selected. Here is also no need to specify a 
    # concentration, because it has been stored in the mixture input file file. It is sufficient to 
    # specify the mixture name, in this case propane/pentane (mixture of 50% mole percent propane and
    # 50% mole percent pentane). 

    Comp = ["propane/pentane"]
    SetFluid( SubLib, Comp)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('freeStanMix, propane/pentane (predefined mix: 50%-50%), P = 5 bar, T = 50 degr.C')
       print ('---------------------------------------------------------------------------------')
 
       P = 5.
       T = 50.

       # To calculate some properties as function of two other independent properties the appropriate 
       # function must be called. In this example additionally the liquid and vapor phase composition 
       # are calculated. For clarity the writing of the error message has been left out, we only print the first return value 
       # of the function, i.e. function('PT', P, T).

       print ('Pressure      = ', Pressure   ('PT', P, T), ' bar      ')
       print ('Temperature   = ', Temperature('PT', P, T), ' degr. C  ')
       print ('Density       = ', Density    ('PT', P, T), ' kg/m3    ')
       print ('Spec. volume  = ', SpecVolume ('PT', P, T), ' m3/kg    ')
       print ('Enthalpy      = ', Enthalpy   ('PT', P, T), ' kJ/kg    ')
       print ('Entropy       = ', Entropy    ('PT', P, T), ' kJ/(kg.K)')
       print ('Vapor quality = ', VaporQual  ('PT', P, T))
       print ()

       x = LiquidCmp('PT', P, T)
       print ( 'Liq phase cmp = propane : ', x[0], ' mole frac ')
       print ( '                pentane : ', x[1], ' mole frac ')
       print ()

       y = VaporCmp('PT', P, T)
       print ( 'Vap phase cmp = propane : ', y[0], ' mole frac ')
       print ( '                pentane : ', y[1], ' mole frac ')
       print ()
          
    # In the third freeStanMix example the same mixture is selected as in the second 
    # example, but now the mole fractions are adapted. To do this you must specify
    # the real number of components in the mixture, the mixture name and concentrations. 
    # In most of the available mixtures the first component comes first in the mixture 
    # name and its concentration must be stored in the first element of the concentration 
    # array; the concentration of the second component must be stored in the second 
    # element, etc. So, concentrations must be stored in the order as the component names
    # appear in the mixture name. 

    Comp = ["propane/pentane"]
    Conc = [ 0.75, 0.25 ]
    SetFluid( SubLib, Comp, Conc)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('freeStanMix, propane/pentane (mix: 75%-25%), P = 5 bar, T = 50 degr.C')
       print ('---------------------------------------------------------------------------------')
 
       P = 5.
       T = 50.

       # To calculate some properties as function of two other independent properties the appropriate 
       # function must be called. In this example additionally the liquid and vapor phase composition 
       # are calculated. For clarity the writing of the error message has been left out, we only print the first return value 
       # of the function, i.e. function('PT', P, T).

       print ('Pressure      = ', Pressure   ('PT', P, T), ' bar      ')
       print ('Temperature   = ', Temperature('PT', P, T), ' degr. C  ')
       print ('Density       = ', Density    ('PT', P, T), ' kg/m3    ')
       print ('Spec. volume  = ', SpecVolume ('PT', P, T), ' m3/kg    ')
       print ('Enthalpy      = ', Enthalpy   ('PT', P, T), ' kJ/kg    ')
       print ('Entropy       = ', Entropy    ('PT', P, T), ' kJ/(kg.K)')
       print ('Vapor quality = ', VaporQual  ('PT', P, T))
       print ()

       x = LiquidCmp('PT', P, T)
       print ( 'Liq phase cmp = propane : ', x[0], ' mole frac ')
       print ( '                pentane : ', x[1], ' mole frac ')
       print ()
          
       y = VaporCmp('PT', P, T)
       print ( 'Vap phase cmp = propane : ', y[0], ' mole frac ')
       print ( '                pentane : ', y[1], ' mole frac ')
       print ()

#--------------------------------------------------------------------------------------------------!
#                                        IF97, water/steam                                         !
#--------------------------------------------------------------------------------------------------!

if IF97_available :

    # IF97 just contains a model for water and steam, so for IF97 the number of components, 
    # component name(s) and concentrations do not matter. So, there is also no need to retrieve 
    # a list of available fluids by calling subroutine GetFluidNames.    

    SubLib = "IF97"
    Comp = ["water/whatever"]
    SetFluid( SubLib, Comp)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('IF97, water, P = 40 bar, h = 3000 kJ/kg')
       print ('----------------------------------------------------------------------------')
 
       P = 40.
       h = 3000.

       # To calculate some properties as function of two other independent properties
       # the appropriate function must be called, exactly like in other examples. In 
       # this example all available functions for IF97 are called and for the input 
       # the pressure and enthalpy are entered. For clarity the writing of the error 
       # message has been left out.

       print ('Pressure      = ', Pressure   ('Ph', P, h), ' bar      ')
       print ('Temperature   = ', Temperature('Ph', P, h), ' degr. C  ')
       print ('Spec. volume  = ', SpecVolume ('Ph', P, h), ' m3/kg    ')
       print ('Density       = ', Density    ('Ph', P, h), ' kg/m3    ')
       print ('Enthalpy      = ', Enthalpy   ('Ph', P, h), ' kJ/kg    ')
       print ('Entropy       = ', Entropy    ('Ph', P, h), ' kJ/(kg.K)')
       print ('Vapor quality = ', VaporQual  ('Ph', P, h))
       print ('Int. energy   = ', IntEnergy  ('Ph', P, h), ' kJ/kg    ')
       print ('Isoc. Heat Cv = ', HeatCapV   ('Ph', P, h), ' kJ/(kg.K)')
       print ('Isob. Heat Cp = ', HeatCapP   ('Ph', P, h), ' kJ/(kg.K)')
       print ('Sound speed   = ', SoundSpeed ('Ph', P, h), ' m/s      ')
       print ('Dyn. Viscos.  = ', Viscosity  ('Ph', P, h), ' Pa.s     ')
       print ('Th. Conduct.  = ', ThermCond  ('Ph', P, h), ' W/(m.K)  ')

#--------------------------------------------------------------------------------------------------!
#                                              GasMix                                              !
#--------------------------------------------------------------------------------------------------!

if GasMix_available :

    SubLib = "GasMix"

    # Using GasMix you can calculate properties of ideal gases and mixtures of ideal gasses. You can 
    # retrieve a list of available components by calling subroutine GetFluidNames. The number of fluids 
    # and an array with fluidnames is returned. 

    FluidName_spec = "long"
    FluidNames = GetFluidNames( FluidName_spec, SubLib)

    print ()
    print ('Available components in GasMix')
    print ('----------------------------------------------------------------------------')

    if GetErrorMsg() == 'No errors' :
       print ('Number of available fluids = ', len(FluidNames))
       print ('The available fluids are: ')
       i = 0
       while i < len(FluidNames):
          # 2-column output 
          if i == len(FluidNames) - 1:
             print (FluidNames[i].rstrip().ljust(40))
          else:
             print (FluidNames[i].rstrip().ljust(40), FluidNames[i+1].rstrip())
          i += 2

    # For GasMix there is just one possibility to define the working fluid. You must specify the 
    # composition by specifying successively the number of components, the components and the 
    # mole fractions. At the moment there is no feature for predefined mixtures.

    # In the example below an air compostion is specified to be the working fluid. The order of the mole 
    # fractions in array Conc must be according the order of the component names in array Comp. It is
    # also possible to define a pure fluid by specifying one component with a mole fraction equaling 1.

    Comp = [ "N2", "O2", "H2O", "CO2" ]
    Conc = [ 0.7801, 0.2094, 0.0102, 0.0003 ]
    SetFluid( SubLib, Comp, Conc)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('GasMix, air, P = 1 bar, T = 15 degr.C')
       print ('----------------------------------------------------------------------------')
 
       P = 1.
       T = 15.
   
       # To calculate some properties as function of two other independent properties
       # the appropriate function must be called, exactly like in other examples. For 
       # clarity the writing of the error message has been left out.
     
       h = Enthalpy( 'PT', P, T)
       s = Entropy ( 'Ph', P, h)

       print ('Enthalpy      = ', h, ' kJ/kg    ')
       print ('Entropy       = ', s, ' kJ/(kg.K)')
       print ('Pressure      = ', Pressure   ('Ps', P, s), ' bar      ')
       print ('Temperature   = ', Temperature('Ph', P, h), ' degr. C  ')
       print ('Spec. volume  = ', SpecVolume ('Ps', P, s), ' m3/kg    ')
       print ('Density       = ', Density    ('Ph', P, h), ' kg/m3    ')
       print ('Isob. Heat Cp = ', HeatCapP   ('PT', P, T), ' kJ/(kg.K)')
       print ('Sound speed   = ', SoundSpeed ('Ph', P, h), ' m/s      ')
       print ('Dyn. Viscos.  = ', Viscosity  ('PT', P, T), ' Pa.s     ')
       print ('Th. Conduct.  = ', ThermCond  ('PT', P, T), ' W/(m.K)  ')
       print ()

#--------------------------------------------------------------------------------------------------!
#                                              RefProp                                             !
#--------------------------------------------------------------------------------------------------!

if RefProp_available :

    SubLib = "RefProp"

    # Using RefProp you can calculate properties of pure fluids and mixtures of 
    # fluids. You can retrieve a list of available components by calling subroutine 
    # GetFluidNames. The number of fluids and an array with fluidnames is returned. 

    # Note that an additional Add-on is required in order to run RefProp or, that, 
    # if you are using MS-Windows, a version of REFPROP obtained from NIST has been 
    # installed. A license for the Add-on can be purchased from the Asimptote web shop. 
    # If the appropriate license is not available on your system then an error message 
    # is displayed.

    FluidName_spec = "long"
    FluidNames = GetFluidNames( FluidName_spec, SubLib)

    print ()
    print ('Available components in RefProp')
    print ('----------------------------------------------------------------------------')

    if GetErrorMsg() == 'No errors' :
       print ('Number of available fluids = ', len(FluidNames))
       print ('The available fluids are: ')
       i = 0
       while i < len(FluidNames):
          # 2-column output 
          if i == len(FluidNames) - 1:
             print (FluidNames[i].rstrip().ljust(40))
          else:
             print (FluidNames[i].rstrip().ljust(40), FluidNames[i+1].rstrip())
          i += 2

    # In RefProp there are two ways to define the working fluid. First you can specify the 
    # composition by specifying successively the number of components, the component names and the 
    # mole fractions, like in GasMix. Second you can define predefined mixtures that are available 
    # in RefProp itself. Of course it is also possible to define a pure fluid by specifying one
    # component with a mole fraction equaling 1.

    # In the example below a mixture of two "refrigerants" R134a and butane is specified to be the 
    # working fluid. The order of the mole fractions in array Conc must be according the order of
    # the component names in array Comp. Instead of specifying the composition there also could 
    # have been chosen to specify the predefined mixture, e.g. R421a, which is mixture of 54% R125
    # and 46% R134a. A predefined mixture must be given as if it is a pure fluid:
    # Comp = [ "R421a" ]
    # SetFluid( "RefProp", Comp) 
        
    Comp = [ "R134a", "butane" ]
    Conc = [ 0.46, 0.54 ]
    SetFluid( SubLib, Comp, Conc)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('RefProp, mixture of R134a and butane, P = 16 bar, T = 25 degr. C')
       print ('----------------------------------------------------------------------------')
 
       P = 16.
       T = 25. 
   
       # To calculate some properties as function of two other independent properties the appropriate 
       # function must be called, exactly like in other examples. Next to various thermodynamic
       # properties also properties related to CFD quantities and transport properties are available, 
       # as well as the liquid and vapor phase compositions (not showed here). For clarity the  
       # writing of the error message has been left out.

       print ('Pressure      = ', Pressure   ('PT', P, T), ' bar      ')
       print ('Temperature   = ', Temperature('PT', P, T), ' degr. C  ')
       print ('Spec. volume  = ', SpecVolume ('PT', P, T), ' m3/kg    ')
       print ('Density       = ', Density    ('PT', P, T), ' kg/m3    ')
       print ('Enthalpy      = ', Enthalpy   ('PT', P, T), ' kJ/kg    ')
       print ('Entropy       = ', Entropy    ('PT', P, T), ' kJ/(kg.K)')
       print ('Vapor quality = ', VaporQual  ('PT', P, T))
       print ('Int. energy   = ', IntEnergy  ('PT', P, T), ' kJ/kg    ')
       print ('Isob. Heat Cv = ', HeatCapV   ('PT', P, T), ' kJ/(kg.K)')
       print ('Isob. Heat Cp = ', HeatCapP   ('PT', P, T), ' kJ/(kg.K)')
       print ('Sound speed   = ', SoundSpeed ('PT', P, T), ' m/s      ')
       print ('Dyn. Viscos.  = ', Viscosity  ('PT', P, T), ' Pa.s     ')
       print ('Th. Conduct.  = ', ThermCond  ('PT', P, T), ' W/(m.K)  ')

       # Note that an additional Add-on is required in order to compute the derivatives. 
       # A license for this Add-on can be purchased from the Asimptote web shop. If the 
       # appropriate license is not available on your system then an error message is 
       # displayed.             
       Alpha( 'PT', P, T)

       if GetErrorMsg() != 'No errors' :
          print ()
          print (GetErrorMsg())
       else:
          print ('Alpha         = ', Alpha      ('PT', P, T), ' m2/s2    ')
          print ('Beta          = ', Beta       ('PT', P, T), ' kg/m3    ')
          print ('Chi           = ', Chi        ('PT', P, T), ' Nm4/kg2.K')
          print ('Fi            = ', Fi         ('PT', P, T), ' 1/K      ')
          print ('Ksi           = ', Ksi        ('PT', P, T), ' kg2/N.m4 ')
          print ('Psi           = ', Psi        ('PT', P, T), ' s2/m2    ')
          print ('Zeta          = ', Zeta       ('PT', P, T), ' kg.K/m3  ')
          print ('Theta         = ', Theta      ('PT', P, T), ' 1/K      ')
          print ('Kappa         = ', Kappa      ('PT', P, T), ' 1/bar    ')
          print ('Gamma         = ', Gamma      ('PT', P, T))
          print ()

#--------------------------------------------------------------------------------------------------!
#                                       Additional features                                        !
#--------------------------------------------------------------------------------------------------!

if freeStanMix_available :
      
   # General names for all libraries are used in both a long and short format. For example n-Hexane 
   # as well as the general names hexane and C6H14 can be used. Fluid names are not case sensitive.

    SubLib  = "freeStanMix"
    Comp = ["c6h14"]
    SetFluid( SubLib, Comp)

    if GetErrorMsg() != 'No errors' :
       print ()
       print (GetErrorMsg())

    else:
       print ()
       print ('freeStanMix, hexane, P = 1 bar, q = 0.5 (SetUnits and AllProps method)')
       print ('----------------------------------------------------------------------------')
 
       P_in = 1.
       q_in = 0.5

       # In this example the use of the AllProps method is showed. This method calculates
       # all available properties in one go. Showing the feature of calculation the 
       # secondary tmd properties in the 2-phase phase region the input specification
       # 'Pq' is used. At the same time unit conversions are shown by defining the
       # unit set to be SI units, with the exception of the pressure, temperature,
       # enthalpy and speed of sound successively to be in bar, degrees Fahrenheit, 
       # Joule per mole and miles per hour.

       # Note that in this example the value of the isobaric heat capacity is not displayed
       # as it is not defined in the two-phase region for pure fluids. 
     
       # Again note that an additional Add-on is required in order to compute the derivatives 
       # alpha, beta,..., gamma. Therefore the values of these properties are not presented
       # here. A license for this Add-on can be purchased from the Asimptote web shop. If 
       # the appropriate license is not available on your system then an error message is 
       # returned by the GetErrorMsg function.           

       SetUnits( "SI", "PerMass", "P,T,h,c", "bar,F,J/mol,mph")        
       if GetErrorMsg() != 'No errors' :
          print ()
          print (GetErrorMsg())

       P, T, v, d, h, s, u, q, x, y, cv, cp, c, alpha, beta, chi, fi, ksi, psi, zeta, theta, kappa, gamma, eta, lamda = AllProps( "Pq", P_in, q_in) 

       print ('Pressure      = ', P.value    , ' bar      ')
       print ('Temperature   = ', T.value    , ' degr.F   ')
       print ('Density       = ', d.value    , ' kg/m3    ')
       print ('Spec. volume  = ', v.value    , ' m3/kg    ')
       print ('Enthalpy      = ', h.value    , ' J/mol    ')
       print ('Entropy       = ', s.value    , ' J/kg.K   ')
       print ('Vapor quality = ', q.value    , ' kg/kg    ')
       print ('Int. energy   = ', u.value    , ' J/kg     ')
       print ('Isoc. Heat Cv = ', cv.value   , ' J/kg.K   ')
       print ('Sound speed   = ', c.value    , ' mph      ')
       print ()
      
       # By specifying blanks the units will be default FluidProp units again.
       SetUnits( " ", " ", " ", " ")        

       print ()
       print ('freeStanMix, hexane, fluid info methods')
       print ('----------------------------------------------------------------------------')

       # Next there is an example of the use of the fluid info methods. Both the use
       # of methods returning a specific property and the method AllInfo returning
       # all info properties in one go are shown.

       M_mol, T_crit, P_crit, T_min, T_max = AllInfo()
       print ('Molar mass    = %g = %g kg/mol   ' % (M_mol.value , Mmol ()))
       print ('T critical    = %g = %g degr.C   ' % (T_crit.value, Tcrit()))
       print ('P critical    = %g = %g kg/mol   ' % (P_crit.value, Pcrit()))
       print ('T min         = %g = %g degr.C   ' % (T_min.value , Tmin ()))
       print ('T max         = %g = %g degr.C   ' % (T_max.value , Tmax ()))

       # Finally, here is an example of the Solve method. First for steam of 40 bar 
       # and 400 C the specific volume is calculated. Thereafter the Solve method 
       # is employed to back-calculate T iteratively from P and v. 
       
       print ()
       print ('IF97 water/steam, Solve method')
       print ('----------------------------------------------------------------------------')

       SetFluid( "IF97")

       P = 40.
       T = 400.
       v = SpecVolume( "PT", P, T)
      
       Target = 2  # i.e. solve T from v = f(P,T); for Target = 1, P is solved from v = f(P,T).
       T_low = 300.
       T_upp = 500.
       T_solved = Solve( "SpecVolume", v, "PT", Target, P, T_low, T_upp)

       if GetErrorMsg() != 'No errors' :
          print ()
          print (GetErrorMsg())
       else:
          print ( 'Pressure                                 = %g bar    ' % P       )
          print ( 'Temperature                              = %g degr. C' % T       )
          print ( 'Spec. volume                             = %g m3/kg  ' % v       )
          print ( 'Temperature iteratively from v = f2(P,T) = %g degr. C' % T_solved)
 
print ()

# At the end of program, you should clean up your memory by deleting the FluidProp objects 
# created in the beginning of the program with the Init_FluidProp call and the FluidProp 
# child objects created with the SetFluid or CreateObject calls. This step is only necessary 
# in Windows and can be skipped for Linux and macOS.

ReleaseObjects()

#========================================================================================= EOF ===#



